
/*
  Test program for PCF8574 I2C I/O expander
 - for use with 4 x 4 keypad.
 
 clear screen works in Minicom, not Arduino IDE serial monitor   
 
 Sparkfun keypad (pin - row or column):
 7  6  5  4  3  2  1
 R1 R2 C2 R3 C0 R0 C1
 
 On port-expander (port - row or column):
 P7 P6 P5 P4 P3 P2 P1 P0
 -  R3 R2 R1 R0 C2 C1 C0
 
 
 */

#include <Wire.h>
//#include <LiquidCrystal.h>

// PCF8574 address: B0100000
// PCA8574A address: B0111000
#define expander B0111000  // Address with three address pins grounded.
// Note that the R/W bit is not part of this address.
// initialize the library with the numbers of the interface pins
//LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

int pbIn = 0;                  // Interrupt 0 is on DIGITAL PIN 2!
int interrupt_counts=0;
boolean terminal_used=false;
int row,column;
byte send_pattern, receive_pattern,test_pattern; 
byte send_pattern_array[]={
  B11101111, B11011111, B10111111, B01111111                        };
byte test_pattern_array[]={
  B00000001, B00000010, B00000100, B00001000                      };
#define intpin 12       // Pin the keyboard interrupt is connected to
char hexaKeys[4][4] = {
  {'D', '#', '0', '*'},
  {'C', '9', '8', '7'},
  {'B', '6', '5', '4'},
  {'A', '3', '2', '1'}
};
char charSav = 'x';
char charIn;
int timeSave;
int timePrev;
int timeMax=0;
int timeMin=32000;

void setup() {
  Wire.begin();
  Serial.begin(9600);

  // initialize the pushbutton pin as an input:
  pinMode(2, INPUT);  
  pinMode(intpin,INPUT_PULLUP);       //Set interrupt pin to input with pullup
  digitalWrite(intpin, HIGH);         //turn on pullup resistor

  //Attach the interrupt to the input pin and monitor for .. 
  //attachInterrupt(pbIn, ISR_pin2, CHANGE)

  Serial.println("greetings!");

//  lcd.begin(20, 2);
//  lcd.print("PCF8574 Test");

}

void loop() {

  if(terminal_used)
  {
    byte ESC=27;
    Serial.print(ESC);
    Serial.print("[2J");
  }
  timeSave=micros();
  charIn=key_scan();
  timePrev=micros()-timeSave;
  if(timePrev > timeMax)
    timeMax=timePrev;
  if(timePrev < timeMin)
    timeMin=timePrev;
  if(charIn != 'x')
  {
    Serial.print("Char= ");
    Serial.print(charIn);
    Serial.print("  Time Max= ");
    Serial.print( timeMax);
    Serial.print("  Time Min= ");
    Serial.println(timeMin);
  }
  //other();
//  delay(300);
}

/*
  void keyscan()
 The columns are connected to the three lowest ports (P2, P1, P0) 
 and the columns have a weak pullup-resistor. We send a 1 to all three
 columns at the same time, we send a 0 on the row for which we want to test, 
 and we send a 1 on the rows that we are not testing. The column on which 
 a key has been pressed will turn from 1 to 0.
 Example: We send 11110111 which means we are testing on row 0 (R0). If the key 
 on row 0 and colum 1 is pressed (number 2 on keypad), then we will read the 
 pattern 11110101. In other words, the second lowest bit has been changed. We 
 repeat the procedure for each existing row on the keypad. In short:
 with port connection [(-) R2 R1 R0 C2 C1 C0 ],
 send 11110111 receive 11110101 means R0 C1 key was pressed.
 Next, we send 11101111 to test row R1, etc.
 */
char key_scan()  
{
  int i=0;
  char cout;
  boolean noCnt= true;

  if (digitalRead(intpin))
  {
    for (i=0; i<4;i++)
    {
      // Try each row. Send 0 on R1 port, the bit on the pressed column
      // will turn from 1 to 0 
      send_pattern=send_pattern_array[i];
      expanderWrite(send_pattern);
      receive_pattern=expanderRead();
  
      if(send_pattern!=receive_pattern)
      {
        row=i;
  
        for (int j=0;j<4;j++)
        {
          test_pattern=test_pattern_array[j]&receive_pattern;     
          if(test_pattern==0)
            column=j;
        }
  
        cout=hexaKeys[row][column];
        charSav=cout;
        noCnt= false;
      }
   }
  }
 if(noCnt)
   {
      charSav='x';
      cout='x';
   }
 return cout;
}


char *btoa(int i)
{

  static char zeros[]="00000000" ;
  char tmpstr[9];

  for(int i=0;i<8;i++)
    zeros[i]='0';

  tmpstr[0] = '\0'; // Initialize the string to 0 length

  itoa (i,tmpstr,2);  
  zeros[8-strlen(tmpstr)]='\0';
  strcat(zeros, tmpstr) ;

  return zeros;  
}


//The interrupt routine
// Note: No print statements in interrupt routine!
void ISR_pin2() 
{
  Serial.println("Interrupt");
  interrupt_counts++;     // a new interrrupt occurred

}


void expanderWrite(byte _data ) {
  Wire.beginTransmission(expander);
  Wire.write(_data);
  Wire.endTransmission();
}

byte expanderRead() {
  byte _data;
  Wire.requestFrom(expander, 1);
  if(Wire.available()) {
    _data = Wire.read();
  }
  return _data;
}

void other()
{  
  Serial.println("Write: B00000111");
  expanderWrite(B00000111);
  Serial.print("Read : ");
  Serial.println(btoa(expanderRead()));
//  lcd.setCursor(0, 1);
//  lcd.print(btoa(expanderRead()));
}

