// TestPB
//
// (c) Copyright Robert Brown 2014-2016 All Rights Reserved.
// Permission is granted for personal and Academic/Educational use only.

// Test Push Buttons

#include <Arduino.h>

// define serial port speed - valid values are 9600, 57600
#define SerialPortSpeed 115200

// define Push Buttons, use voltage divider network for two push button switches using A0, use software debouncing
#define RAswitchesPin  A3   // push button switches wired to A0 via resistor divider network
#define DECswitchesPin  A2   // push button switches wired to A0 via resistor divider network
int PBValRA = 0;              // holds state of pushbutton switches
int PBVal=0;
int PBValSav=0;
int PBValSaveRA = 0;
int PBValDEC = 0;              // holds state of pushbutton switches
int PBValSaveDEC = 0;
int aval=-1;
int decval=-1;
int raval=-1;


//**************************************************************************************
// Push button Routines
//**************************************************************************************
// read the push button switches and return state of switches
// Inputs: None
// Outputs: 1 = SW1 ON AND SW2 OFF, 2 = SW2 ON AND SW1 OFF, 3 = SW1 ON and SW2 ON, 0 = OFF
int readpbswitches()
{
  //  sw1 (681) 650-720, sw2 (338) 310-380, sw1 and sw2 (509) 460-530
  int val = 0;                       // variable to store the read value
  int tmpval=0;
//***********************************************************************************
// Check Dec switches
//***********************************************************************************
  val = analogRead(RAswitchesPin);          // read the input pin
  aval=val;
  if ( val >= 650 && val <= 720 )
  {
    tmpval= 1;                        // toggle sw1 ON and SW2 OFF
  }
  else if ( val >= 460 && val <= 530 )
  {
    tmpval= 3;                        // toggle sw1 and sw2 ON
  }
  else if ( val >= 310 && val <= 380 )
  {
    tmpval= 2;                        // toggle sw2 ON and SW1 OFF
  }
  else if ( val < 310 )
  {
    tmpval= 4;                        // toggle sw2 ON and SW1 OFF
  }
  else if ( val > 720 )
  {
    tmpval= 0;                        // toggle sw2 ON and SW1 OFF
  }
  else tmpval= 0;                     // switches are OFF
//***********************************************************************************
// Check Dec switches
//***********************************************************************************
  
  tmpval=tmpval*10;
  val = analogRead(DECswitchesPin);          // read the input pin
  aval=val;
  if ( val >= 650 && val <= 720 )
  {
    val=tmpval+1;
    return val;                        // toggle sw1 ON and SW2 OFF
  }
  else if ( val >= 460 && val <= 530 )
  {
    val=tmpval+3;
    return val;                        // toggle sw1 and sw2 ON
  }
  else if ( val >= 310 && val <= 380 )
  {
    val=tmpval+2;
    return val;                        // toggle sw2 ON and SW1 OFF
  }
  else if ( val < 310 )
  {
    val=tmpval+4;
    return val;                        // toggle sw2 ON and SW1 OFF
  }
  else if ( val > 720 )
  {
    val=tmpval;
    return val;                        // toggle sw2 ON and SW1 OFF
  }
  else
  {
    val=tmpval;
    return val;                     // switches are OFF
  }
}

void GetPBSwitch ()
{
  PBVal = readpbswitches();
  if ( PBVal != 0)
  {
    // if its a genuine button press
    delay(20);
    PBVal = readpbswitches();
  }
    if(PBVal != PBValSav)
    {
      PBValSav=PBVal;
      PBValRA=PBVal/10;
      PBValDEC=PBVal-PBValRA*10;
      Serial.print("PBVal= ");
      Serial.print(PBVal);
      Serial.print(" PBValRA= ");
      Serial.print(PBValRA);
      Serial.print(" PBValDEC= ");
      Serial.println(PBValDEC);
      
      if(PBValRA != PBValSaveRA )
      {
        // now check the pbval using a switch for 1 2 and 3
        switch ( PBValRA )
        {
            case 1:                        // toggle sw1 is ON and 2 is off
              Serial.println("RA LEFT is pressed");
              break;
            case 2:                        // toggle sw2 is ON and SW1 is OFF
              Serial.println("RA RIGHT is pressed");
               break;
            case 3:                        // toggle sw1 and sw2 are ON
              Serial.println("Both RA Buttons are pressed");
              break;
            case 4:                        // Home Button
              Serial.println("ENTER Button Pressed");
              break;
            case 0:                        // toggle sw1 and sw2 are ON
              Serial.println("RA Buttons Clear");
              break;
            default:
              Serial.println("RA No button is pressed");
              break;
        } // end of switch
    
       PBValSaveRA=PBValRA;
      }
    if ( PBValDEC != PBValSaveDEC)
    {
      // now check the pbval using a switch for 1 2 and 3
      switch ( PBValDEC )
      {
          case 1:                        // toggle sw1 is ON and 2 is off
            Serial.println("DEC UP is pressed");
            break;
          case 2:                        // toggle sw2 is ON and SW1 is OFF
            Serial.println("DEC DOWN is pressed");
            break;
          case 3:                        // toggle sw1 and sw2 are ON
            Serial.println("Both DEC Switches are pressed");
            break;
          case 4:                        // Home Button
            Serial.println("EXIT Button Pressed");
            break;
          case 0:                        // toggle sw1 and sw2 are ON
            Serial.println("DEC Buttons Clear");
            break;
          default:
            Serial.println("DEC No button is pressed");
            break;
      } // end of switch
      PBValSaveDEC=PBValDEC;
      }
  } // end of Push Button check}
}
// Setup
void setup()
{
  Serial.begin(SerialPortSpeed);

  // read state of pb switches
  pinMode( RAswitchesPin, INPUT_PULLUP );
  pinMode( DECswitchesPin, INPUT_PULLUP );
  digitalWrite(RAswitchesPin, HIGH);
  digitalWrite(DECswitchesPin, HIGH);
}

// Main Loop
void loop()
{
  // SPECIAL NOTE START
  // check pushbutton switches
  // if you are not using the Push Button switches this part of code MUST BE DELETED
    GetPBSwitch ();
  // SPECIAL NOTE END
  delay(100);
}
