/*********************

Example code for the Adafruit RGB Character LCD Shield and Library

This code displays text on the shield, and also reads the buttons on the keypad.
When a button is pressed, the backlight changes color.

**********************/

// include the library code:
#include <Wire.h>
#include <Adafruit_RGBLCDShield.h>
#include <utility/Adafruit_MCP23017.h>


// The shield uses the I2C SCL and SDA pins. On classic Arduinos
// this is Analog 4 and 5 so you can't use those for analogRead() anymore
// However, you can connect other I2C sensors to the I2C bus and share
// the I2C bus.
Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

// These #defines make it easy to set the backlight color
#define RED 0x1
#define YELLOW 0x3
#define GREEN 0x2
#define TEAL 0x6
#define BLUE 0x4
#define VIOLET 0x5
#define WHITE 0x7
#define Buzzer 12

int buzfreq=500;

void setup() {
  // Debugging output
  Serial.begin(9600);
  // set up the LCD's number of columns and rows: 
  lcd.begin(16, 2);

  // Print a message to the LCD. We track how long it takes since
  // this library has been optimized a bit and we're proud of it :)
  int time = millis();
  lcd.print("Hello, world!");
  time = millis() - time;
  Serial.print("Took "); Serial.print(time); Serial.println(" ms");
  lcd.setBacklight(WHITE);
  pinMode(12,OUTPUT);
  digitalWrite(12,LOW);     // Turn buzzer off
}

uint8_t i=0;
void loop() {
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  lcd.setCursor(0, 1);
  // print the number of seconds since reset:
  lcd.print(millis()/1000);

  uint8_t buttons = lcd.readButtons();

  if (buttons) {
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print(buttons);
    lcd.print(" ");
    switch (buttons){
    case 1:
      Serial.print("buzfreq=");
      Serial.println(buzfreq);
      buzz(Buzzer, buzfreq, 500); // buzz the buzzer at 500Hz for 500 milliseconds
      buzfreq=buzfreq+500;
      if(buzfreq > 5500)
          buzfreq=500;
      
      lcd.print("SELECT Blue");
      lcd.setBacklight(BLUE);
      break;
    case 2:
      buzz(Buzzer, 1000, 500); // buzz the buzzer at 1000Hz for 500 milliseconds
      lcd.print("RIGHT Teal");
      lcd.setBacklight(TEAL);
      break;
    case 4:
      buzz(Buzzer, 2000, 500); // buzz the buzzer at 2000Hz for 500 milliseconds
      lcd.print("DOWN Yellow");
      lcd.setBacklight(YELLOW);
      break;
    case 8:
      buzz(Buzzer, 3000, 500); // buzz the buzzer at 3000Hz for 500 milliseconds
      lcd.print("UP Red");
      lcd.setBacklight(RED);
      break;
    case 16:
      buzz(Buzzer, 4000, 500); // buzz the buzzer at 1000Hz for 500 milliseconds
      lcd.print("LEFT Green");
      lcd.setBacklight(GREEN);
      break;
    default:
     buzz(Buzzer, 200, 500); // buzz the buzzer at 200Hz for 500 milliseconds
     lcd.print("Default White");
     lcd.setBacklight(WHITE);
  }
 }
}

//**************************************************************************************
// Buzzer Routine
//**************************************************************************************
// Inputs:
//         targetPin - pin on the Arduino the buzzer is attached to
//         frequency - frequency in hz. for the buzzer sound
//         length    - time in msec for the buzzer to sound

void buzz(int targetPin, long frequency, long length) 
{
  long delayValue = 1000000 / frequency / 2; // calculate the delay value between transitions
  //// 1 second's worth of microseconds, divided by the frequency, then split in half since
  //// there are two phases to each cycle
  long numCycles = frequency * length / 1000; // calculate the number of cycles for proper timing
  //// multiply frequency, which is really cycles per second, by the number of seconds to
  //// get the total number of cycles to produce
//  if (Configs.enBUZZ) {
    for (long i = 0; i < numCycles; i++) { // for the calculated length of time...
      digitalWrite(targetPin, HIGH); // write the buzzer pin high to push out the diaphram
      delayMicroseconds(delayValue); // wait for the calculated delay value
      digitalWrite(targetPin, LOW); // write the buzzer pin low to pull back the diaphram
      delayMicroseconds(delayValue); // wait againf or the calculated delay value
    }
//  }
}

