// basic test program for CNC Controller Board Stepper Motors
// rotates motor one full rotation then the reverse direction with 2s delay between moves

#include <Arduino.h>

// Stepper Motor stuff, control pins for DRV8825 board
#define RAdrvDIR     2     //RA Drive Direction
#define RAdrvSTP    5     //RA Drive Step
#define DECdrvDIR     3       //Dec Drive Direction
#define DECdrvSTP    6       //Dec Drive Step

// ENABLE - Setting CoilPwr controls the disable or enable state of the EASYDRIVER board outputs
int boardstate;
#define OUTPUTENABLED 1
#define OUTPUTDISABLED 0
#define drvENABLE  8        // For DRV8825 and Easydriver boards, wire /ENABLE pin to D8 input

// stepontime - time in microseconds that coil power is ON for one step, board requires 2us pulse
int stepontime = 5;
// motorSpeed - time in milliseconds of delay between stepper pulses
int motorSpeedSlow = 15;
int motorSpeedMed = 8;
int motorSpeedFast = 3;
int motorSpeed = motorSpeedMed;
boolean emSTOP=false;
boolean drvState;
struct config_t
{
  int Zfield;                 // If the first character is 99 that says the data has been written previously and is valid
  // If the first character is an "F" then the data location has been written, but is not valid
  unsigned int eeAccess;        // Number of times the data has been accessed
  int stepENON=800;                 // delay for Enable to be on before Stepping
  int stepENOFF=800;                // delay for Enable to stay on after step
  int stepontime;               //time in microseconds that coil power is ON for one step, board requires 2us pulse
  boolean RAstepDIR=false;            //Direction of RA step false-clockwise true-anticlockwise
  boolean DECstepDIR=false;           //Direction of DEC step false-clockwise true-anticlockwise
  boolean enBUZZ;               //Enable Buzzer 0-Buzzer Off  1-Buzzer allowed
  long roundtime;               //Number of steps in one revolution
  int Sdn;                      // # steps for slow
  int Sup;                      // # steps for fast
  int slewDelay;                // delay in usec between steps when slewing
  unsigned int minScale;        // number from 0-65535 that is the scaling factor for the stepper driver for fast push button
  boolean EQN;                  // True = EQ North  False=EQ South
  byte guideSpeed;              // 0= 2X, 1=4X, 2=8X, 3=20X
  byte drvRate;                 // 0=Sidereal, 1=Solar 2=Lunar
  byte keys;                    // 0=Normal 1=Reverse RA 2= Reverse DEC 3=Reverse RA and DEC, 4=Flip RA/DEC 5=Flip+Rev RA, 6= Flip+ Rev DEC, 7=Flip+RevRA+RevDec
  byte blColor;                 // 1= Red, 2= Green, 3=Yellow, 4=Blue 5=Violet 6=Teal 7=White
} Configs;
// Timing variables
long roundclk = 0;
int roundcnt = 0;
int stepcnt = 1;
int roundsave = 0;

//**************************************************************************************
// Stepper motor routines
//**************************************************************************************
// enable the stepper motor outputs - coil power on
void STEPenable() {
  digitalWrite(drvENABLE, LOW);
  drvState = true;
}
void STEPdisable() {
  digitalWrite(drvENABLE, HIGH);
  drvState = false;
}
//**************************************************************************************
// Routine to move stepper anticlockwise
void anticlockwise(byte drvDIR, byte drvSTP) {
  if (!emSTOP) {
    digitalWrite(drvDIR, LOW );
    digitalWrite(drvENABLE, LOW);
    delayMicroseconds(Configs.stepENON);
    digitalWrite(drvSTP, HIGH );
    delayMicroseconds(Configs.stepontime);
    digitalWrite(drvSTP, LOW );
    delayMicroseconds(Configs.stepENOFF);
    digitalWrite(drvENABLE, HIGH);
    if(drvDIR == RAdrvDIR){
      if (Configs.RAstepDIR)
        roundclk = roundclk + stepcnt;
      else
        roundclk = roundclk - stepcnt;
    }
  }
}
//**************************************************************************************
// Routine to move stepper clockwise
void clockwise(byte drvDIR, byte drvSTP) {
  if (!emSTOP) {
    digitalWrite(drvDIR, HIGH );
    digitalWrite(drvENABLE, LOW);
    delayMicroseconds(Configs.stepENON);
    digitalWrite(drvSTP, HIGH );
    delayMicroseconds(Configs.stepontime);
    digitalWrite(drvSTP, LOW );
    delayMicroseconds(Configs.stepENOFF);
    digitalWrite(drvENABLE, HIGH);
    if(drvDIR == RAdrvDIR){
      if (Configs.RAstepDIR)
        roundclk = roundclk - stepcnt;
      else
        roundclk = roundclk + stepcnt;
    }
  }
}


// Setup
void setup() {
  // initialize serial for ASCOM
  Serial.begin(9600);

  pinMode(  RAdrvDIR, OUTPUT );
  pinMode(  RAdrvSTP, OUTPUT );
  pinMode(  DECdrvDIR, OUTPUT );
  pinMode(  DECdrvSTP, OUTPUT );

  // set direction and step to low
  digitalWrite( RAdrvDIR, 0 );
  digitalWrite( RAdrvSTP, 0 );
  digitalWrite( DECdrvDIR, 0 );
  digitalWrite( DECdrvSTP, 0 );

  boardstate = OUTPUTENABLED;
  pinMode( drvENABLE, OUTPUT );    // enable the driver board
  STEPdisable();
}

// Main Loop
void loop() {
  Serial.println("RA Clockwise 200 steps - Full steps");
  for ( int lp = 0; lp < 200; lp++ )
  {
    clockwise(RAdrvDIR,RAdrvSTP);
    delay(10);
  }
  Serial.println("DEC Clockwise 200 steps - Full steps");
  for ( int lp = 0; lp < 200; lp++ )
  {
    clockwise(DECdrvDIR,DECdrvSTP);
    delay(10);
  }
  Serial.println("Wait 2s");
  delay(2000);
  Serial.println("RA Anti-Clockwise 200 steps - Full steps");
  for ( int lp = 0; lp < 200; lp++ )
  {
    anticlockwise(RAdrvDIR,RAdrvSTP);
    delay(10);
  }
  Serial.println("DEC Anti-Clockwise 200 steps - Full steps");
  for ( int lp = 0; lp < 200; lp++ )
  {
    anticlockwise(DECdrvDIR,DECdrvSTP);
    delay(10);
  }
  Serial.println("Wait 2s");
  delay(2000);
}


