//**************************************************************************************
// For use with the CG-4 Controller that has an Arduino Nano and a DRV8825 Stepper Driver Module     

// Buzzer is a piezo crystal buzzer
//
// (c) copyright Z-Field Learning, LLC. 2017-2018
//
// Version 1.01 changes
//  Added heartbeat to Nano LED to be able to see East (1 pulse per second) or West (2 seconds per pulse)
//  Added ablility to change drive tracking rates from serial port
//  Fixed EEPROMAnything error
//  10/28/19 Fixed problem with Command 70h, 112d p command that did not have the number of bytes set in slength
// 
// include the library code:
#include <Wire.h>
#include <math.h>
#include <EEPROM.h>                   // for eeprom storage
#include <avr/pgmspace.h>

#include <Adafruit_RGBLCDShield.h>
#include <utility/Adafruit_MCP23017.h>
#include <EEPROMAnything.h>           // for eeprom structure storage
#define EEPROMSIZE 1000             // ATMEGA328P 1024 EEPROM
char VerNum[] = "Version 1.01    ";
#define CurntConf 97
#define ForceConfigs 0

//**************************************************************************************
// RA Stepper Motor control pins for DRV8825 board
#define RAdrvDIR     2
#define RAdrvSTP    5
//**************************************************************************************
// DEC Stepper Motor control pins for DRV8825 board
#define DECdrvDIR     3
#define DECdrvSTP    6
// drvENABLE - The enable pin on the DRV8825 turns on/off the current to the stepper Only one ENABLE that controls both DEC and RA
#define drvENABLE  8        
//**************************************************************************************
// define non saved stepper Flags
boolean emStop=false;                         //boolean emSTOP = false ;                      // Stop false= Run, true=STOP
boolean DECenable=false;                      // Enable DEC slow motion buttons
uint16_t drvScale[3];                         // Timer1 Scale factors for Sidereal, Solar and Lunar rates. These are 100x values
//uint16_t drvScale[]{18699,18750,19152};       // Timer1 Scale factors for Sidereal, Solar and Lunar rates. These are 100x values
uint8_t RASpeedspos[] {50,33,25,10,6,3,1};    // Array for Scale Factors for RA Positive speeds
uint8_t RASpeedsneg[] {255,100,0,17,7,3,1};   // Array for Scale Factors for RA Negative speeds
uint8_t DECSpeeds[] {0,112,75,37,19,3,1};     // Array for Scale Factors for Declination speeds
uint8_t DECSpeed;                             // Current DEC Speed pointer (1-6)
uint8_t DECCnt;                               // Counter for DEC drive
uint8_t RAposSpeed;                           // RA SPeed Pointer
uint8_t RAnegSpeed;                           // RA SPeed Pointer
uint8_t RACnt;                                // Counter for RA drive
boolean RADirFlag;                            // Current direction of the RA drive
boolean DECDirFlag;                           // true=North  false=South
boolean dirSpeed;                             // Temporary storage of current direction for remote speeds
boolean SlewEnabled=false;                    // Flag to allow for slewing from PC Enabled=true, Disabled=false
boolean RAturbo=false;                        // Flag to go to fast mode if both direction keys are pressed
boolean DECturbo=false;                       // Flag to go to fast mode if both direction keys are pressed
uint8_t inbutsav;                             // storage location for in button
boolean Slewing=false;                        // true= movement in progress  false=motor not moving
boolean GoToEN=false;                         // GoTo in progress
uint32_t GoToRA;                              // Steps for RA to take for GOTO
uint32_t GoToDEC;                             // Steps for RA to take for GOTO
uint8_t RAGuideRate=128;                      // the RASpeed for guide rate is derived from GuideRate/256
uint8_t DECGuideRate=128;                     // the RASpeed for guide rate is derived from GuideRate/256
int RASstp;
int DECSstp;
boolean HalfStep=true;                        // HalfStep is used in ISR to divide the clock by 2
boolean RAENT=false;                          // RAENT and DECENT are flags to say if they need sepping
boolean DECENT=false;
boolean EastSide;                 // Side of the meridian the scope is on East=true West=false

//**************************************************************************************
// Time variables
long offms=995;                                                 //millis() in one second
long nowms;
long prevms;
boolean HB=false;                       // Flash LED on Nano to show it is working
uint8_t HB2=1;                          // USed to count heartbeats, 1= East, 2=West
#define HBLED 13                        // Set port of LED on Nano
uint8_t secPrev;
uint8_t minPrev;
uint8_t seconds=0;
uint8_t minutes=0;
uint8_t hours=18;
uint8_t DST;
uint8_t TZ=0xFA;                         // Time Zone (if >256 then negative)
uint8_t month=2;
uint8_t day=20;
uint8_t year=18;
uint8_t SidHrs;
uint8_t SidMin;
uint8_t SidSec;
uint8_t LineFlag=0;                       //Flag to show what character we're pointing at in Time, Lat and Long set
boolean TimeSet=false;                    // true if in Time Set mode
boolean InitAll=false;                    //Flag to initialize date,time,lat,long
boolean TimeDate=false;                   //Flag to set both date and time
boolean LatLong=false;                    //Flag to set both Lat and Long
boolean RaDec=false;                      //Flag to set both RA  and DEC
uint8_t TimeLocValid=15;                  // Flags for when time and location are entered
uint8_t RAHrs=0;
uint8_t RAMin=0;
uint8_t RASec=0;
uint8_t DECDeg=0;
uint8_t DECMin=0;
uint8_t DECSec=0;
uint8_t DECsgn=0;


//**************************************************************************************
//PC Communications Variables
uint8_t slength[] {1,0,0,10,0,0,1,0,0,9,0,1,2,1,1,1,8,8,   //63-80 (?-P)
                   0,10,10,2,0,1,9,0,0,1,0,0,0,0,0,0,0,0,   //81-98 {Q-a)
                   0,0,1,0,0,1,0,0,0,1,1,0,0,1,0,18,18,1,0,1,1,0,0,1  //99-122 (c-z)
                };
uint8_t receivedChars[20];                                   //Received bytes buffer
uint16_t tempI;                                              //Temporary Int storage
uint8_t numChars;
boolean firstChar=true;
boolean newData = false;
uint32_t MTimeOut=0;
uint32_t STimeOut=0;
boolean SerMove=0;
uint8_t PCcmd=0;                                        // Put first byte (PC Command) in here
uint8_t PCnum=0;                                        // How many commands have been sent by PC
uint8_t byte1=1;
uint8_t byte0=0;
union Pos
{
  uint32_t pos;                             // Position structure
  uint8_t b[4];
  uint16_t wrd[2];

};
  union Pos DECp, RAp, RAt, DECt, SidTime, ValP, Valtmp;              
  // Define Structures for current position:DEC (DECp) and RA (RAp)
  // Define Structures for temporary position:DEC (DECt) and RA (RAt)
  // Define Structures for current Sidereal Time (SidTime)
// SidTime.pos=3B300500                         // this shows how they are stored in memory (from a program run)
// wrd [0]= 500  Word [1]= 3B30
// b [0]=0  [1]=5 [2]=30 [3]=3B

uint8_t RAvalH;
uint8_t RAvalM;
uint8_t RAvalS;
uint8_t RAvalD;
uint8_t DECvalD;
uint8_t DECvalM;
uint8_t DECvalS;
uint8_t DECNeg;                                 // if DECNeg > 128, dec is negative
char iString[4];
//**************************************************************************************
// define objects for Menu
uint8_t prevButton=0;                              // Setting for previous Button push
uint8_t menuPTR;                                 // Pointer to the current Menu Item
uint8_t menuPrev=0;                              //Previous menu item to tell if things have changed
uint8_t topPTR;                                  // Pointer to what menu we are in
uint8_t subPTR;                                 // pointer to what submenu item we are at
uint8_t inMenu = 0;                       // are we in a menu?  True= Yes, False= No
uint8_t butPress=0;
// boolean dontExit=false;
String clrLine="                ";            // 16 spaces to clear a line
#define degChar 223                          // Degree symbol
#define dotChar 165                          // Degree symbol

const char menuItem1[] PROGMEM = "Exit";
const char menuItem2[] PROGMEM = ".5X";
const char menuItem3[] PROGMEM = "2X";
const char menuItem4[] PROGMEM = "4X";
const char menuItem5[] PROGMEM = "8X";
const char menuItem6[] PROGMEM = "16X";
const char menuItem7[] PROGMEM = "Sidereal";
const char menuItem8[] PROGMEM = "Solar";
const char menuItem9[] PROGMEM = "Lunar";
const char menuItem10[] PROGMEM = "Buzzer ON";
const char menuItem11[] PROGMEM = "Buzzer OFF";
const char menuItem12[] PROGMEM = "BackLight OFF";
const char menuItem13[] PROGMEM = "Red";
const char menuItem14[] PROGMEM = "Green";
const char menuItem15[] PROGMEM = "Yellow";
const char menuItem16[] PROGMEM = "Blue";
const char menuItem17[] PROGMEM = "Violet";
const char menuItem18[] PROGMEM = "Teal";
const char menuItem19[] PROGMEM = "White";
const char menuItem20[] PROGMEM = "Sidereal Time";
const char menuItem21[] PROGMEM = "Set Location";
const char menuItem22[] PROGMEM = "Set Time/Date";
const char menuItem23[] PROGMEM = "EQN";
const char menuItem24[] PROGMEM = "EQS";
const char menuItem25[] PROGMEM = "Reset defaults";
const char menuItem26[] PROGMEM = "Code Version";
const char menuItem27[] PROGMEM = "Enter Coords";
const char menuItem28[] PROGMEM = "Sync";
const char menuItem29[] PROGMEM = "Goto";
const char menuItem30[] PROGMEM = "StopMotors";

const char* const MenuTable [] PROGMEM ={
menuItem1, menuItem2, menuItem3, menuItem4, 
menuItem5, menuItem6, menuItem7, menuItem8, 
menuItem9, menuItem10, menuItem11, menuItem12, 
menuItem13, menuItem14, menuItem15, menuItem16, 
menuItem17, menuItem18, menuItem19, menuItem20, 
menuItem21, menuItem22, menuItem23, menuItem24,
menuItem25, menuItem26, menuItem27, menuItem28,
menuItem29, menuItem30
};
#define FLExit 0
#define FL_5X 1
#define FL2X 2
#define FL4X 3
#define FL8X 4
#define FL20X 5
#define FLSid 6
#define FLSolar 7
#define FLLunar 8
#define FLBuzzON 9
#define FLBuzzOFF 10
#define FLOff 11
#define FLRed 12
#define FLGreen 13
#define FLYellow 14
#define FLBlue 15
#define FLViolet 16
#define FLTeal 17
#define FLWhite 18
#define FLSidTime 19
#define FLSetLocation 20
#define FLSetTimeDate 21
#define FLEQN 22
#define FLEQS 23
#define FLResetdefaults 24
#define FLDispVer 25
#define FLEnterCoords 26
#define FLSync 27
#define FLGoto 28
#define FLStopMotors 29

char buffer[16];
#define topMenuMax 6
const char* topMenu [] = {
"Menu","Speed", "Rate", "Display","Configure Mount","GOTO"
};
uint8_t subMenuCnt[6] ={1,5,3,10,7,4};
uint8_t subMenuStart[6] = {0,1,6,9,19,26};
//00 0000 0000 0000 0001 0010 0110 0000

long itemEndef  = 0x00001260;

#define numItems 29
boolean itmPrnt=true;                                         // false=suspend printing of that item.
//**************************************************************************************
// The LCD shield uses the I2C SCL and SDA pins. On classic Arduinos
// this is Analog 4 and 5 so you can't use those for analogRead() anymore
// However, you can connect other I2C sensors to the I2C bus and share
// the I2C bus.

Adafruit_RGBLCDShield lcd = Adafruit_RGBLCDShield();

// These #defines make it easy to set the backlight color
#define BLOFF 0x0
#define RED 0x1
#define YELLOW 0x3
#define GREEN 0x2
#define TEAL 0x6
#define BLUE 0x4
#define VIOLET 0x5
#define WHITE 0x7
uint8_t buttons;
//**************************************************************************************
// define objects for buzzer
#define Buzzer 12
//**************************************************************************************
// define objects for errors
uint8_t errnum;
boolean buzon=false;                        // buzon= true continue to beep  buzon=false don't beep
const char errorno0[] PROGMEM = "  Menu    ";
const char errorno1[] PROGMEM = "MenuTimOut";
const char errorno2[] PROGMEM = "Bad Menu  ";
const char errorno3[] PROGMEM = "PC Change ";
const char errorno4[] PROGMEM = "PTO ";
const char errorno5[] PROGMEM = "Track Err ";
const char errorno6[] PROGMEM = "Track Off ";
const char errorno7[] PROGMEM = "Merid Xing";
const char errorno8[] PROGMEM = "SetDateTim";
uint8_t errFreq[9] = {5,1,2,1,1,25,25,25,25};
uint8_t errLen[9] = {2,1,5,1,1,2,2,2,2};

const char* const ErrTable [] PROGMEM ={
errorno0, errorno1, errorno2, errorno3, 
errorno4, errorno5, errorno6, errorno7,
errorno8
};
//**************************************************************************************
// define structure of data stored in eeprom that will be read each time
struct config_t
{
  int Zfield;                         // If the first character is CurntConf that says the data has been written previously and is valid
  // If the first character is an "F" then the data location has been written, but is not valid
  unsigned int eeAccess;              // Number of times the data has been accessed
  boolean EQN;                  // True = EQ North  False=EQ South
  uint16_t SidRate;                    // Sidereal Rate value
  uint16_t SolRate;                    // Solar Rate value
  uint16_t LunRate;                    // Lunar Rate value
  boolean RAEQN;                      //Normal Direction of RA in EQN moves drive CounterClockwise
  boolean DECEQN;                     //Normal Direction of DEC UP button moves Positive North when scope is on the East Side of mount
  long itemEnable;                     // enabled flags for each menu item.
  uint8_t blColor;                     // 1= Red, 2= Green, 3=Yellow, 4=Blue 5=Violet 6=Teal 7=White
  boolean enBUZZ;               //Enable Buzzer 0-Buzzer Off  1-Buzzer allowed
  uint8_t guideSpeed;              // 1=0.5X 2= 2X, 3=4X, 4=8X, 5=16X 6=40x (slew speed only)
  uint8_t keys;                    // 0=Normal 1=Reverse RA 2= Reverse DEC 3=Reverse RA and DEC, 4=Flip RA/DEC 5=Flip+Rev RA, 6= Flip+ Rev DEC, 7=Flip+RevRA+RevDec

  //**************************************************************************************
  // Locaton variables
  uint8_t LatDeg;                      //Latitude Degrees
  uint8_t LatMin;                      //Latitude Minutes
  uint8_t LatSec;                      //Latitude Seconds
  uint8_t LatDir;                      //Latitude Direction 0=North 1=South
  uint8_t LongDeg;                     //Longitude Degrees
  uint8_t LongMin;                     //Longitude Minutes
  uint8_t LongSec;                     //Longitude Seconds
  uint8_t LongDir;                     //Longitude Direction 0=East 1=West
  uint32_t blank3;                     // place holder
  uint32_t blank2;                     // place holder
  uint32_t blank1;                     // place holder
  int EEPROMWrites;
} Configs;

boolean DECStepDir;           //Normal Direction of DEC UP button moves Positive North when scope is on the East Side of mount
boolean RAStepDir;            //Normal Direction of RA in EQN moves drive CounterClockwise
uint8_t guideSpdSav;          // Location to save original guide speed during PC guiding
uint8_t drvRate;              // 0=Sidereal, 1=Solar 2=Lunar
int slewDelay;                // delay in usec between steps when slewing
int stepENON;                 // delay for Enable to be on before Stepping
int stepENOFF;                // delay for Enable to stay on after step
int stepontime;               //time in microseconds that coil power is ON for one step, board requires 2us pulse
int currentaddr;              // will be address in eeprom of the data stored
bool writenow=false;          // should we update values in eeprom
long previousMillis;            // used as a delay whenever the EEPROM settings need to be updated
bool found;                   // did we find any stored values?
int datasize;                 // will hold size of the struct Configs
int nlocations;               // number of storage locations available in EEPROM
#define EW1 9                 // Pin for East/West switch
int8_t Eval1;
int8_t Eval2;
int8_t EvalPrev=0;



//**************************************************************************************
// Menu items and their flag bit in itemEnable
//  Exit -0x1
//  .5X -0x2
//  2X -0x4
//  4X -0x8
//  8X -0x10
//  16X -0x20
//  Sidereal -0x40
//  Solar -0x80
//  Lunar -0x100
//  Buzzer ON -0x200
//  Buzzer OFF -0x400
//  Off -0x800
//  Red -0x1000
//  Green -0x2000
//  Yellow -0x4000
//  Blue -0x8000
//  Violet -0x10000
//  Teal -0x20000
//  White -0x40000
//  Sidereal Time -0x80000
//  Set Location -0x100000
//  Set Time/Date -0x200000
//  EQN -0x400000
//  EQS -0x800000
//  Reset Defaults -0x1000000
//  Not assigned -0x2000000
//  Enter Coords -0x4000000
//  Sync -0x8000000
//  GoTO -0x10000000
//  Stop Motors -0x20000000

//**************************************************************************************
// define various variables
boolean Debug =false;                 // Debug: true print debug statements  false- dont print
const String programVersion = "1.0";

//**************************************************************************************
// Stepper motor routines
//**************************************************************************************

void timer1Setup()
{
  int pre=1;
  uint8_t cs10B = 0;
  uint8_t cs11B = 0;
  uint8_t cs12B = 0;
  if (pre & B100) cs12B = 1;
  if (pre & B010) cs11B = 1;
  if (pre & B001) cs10B = 1;
  cli();//stop interrupts
  //set timer1 interrupt at 16Hz
  TCCR1A = 0;// set entire TCCR1A register to 0
  TCCR1B = 0;// same for TCCR1B
  TCNT1  = 0;//initialize counter value to 0
  // set compare match register
  
  OCR1A = drvScale[drvRate];// stepScale= (16*10^7) / (steps/sec*Configs.stepPRE) - 1 (must be <65536)
  // turn on CTC mode
  TCCR1B |= (1 << WGM12);
  // Set CS10 and CS11 bits for 64 prescaler
  TCCR1B |= (cs12B << CS12) | (cs11B << CS11) | (cs10B << CS10);
  // enable timer compare interrupt
  TIMSK1 |= (1 << OCIE1A);
  sei();//allow interrupts
}


//**************************************************************************************
// Timer 1 Interrupt Rounte to step the stepper motor
//**************************************************************************************
ISR(TIMER1_COMPA_vect) {    //timer1 interrupt 16Hz Stepper 1 step
  boolean RADirTmp;
  boolean DECDirTmp;
// HalfStep is used in ISR to divide the clock by 2
// GoRA and GoDEC tell the ISR to enable the steppers and step.
  if(HalfStep)
  {
    // enable steps and step here
    if(RAENT)
       digitalWrite(RAdrvSTP, 1 );
    if(DECENT)
       digitalWrite(DECdrvSTP, 1 );
    delayMicroseconds(stepontime);
    if(RAENT)
       digitalWrite(RAdrvSTP, 0 );
    if(DECENT)
       digitalWrite(DECdrvSTP, 0 );
    HalfStep=false;
  }
  else
  {
    digitalWrite(drvENABLE, HIGH);
    RAENT=false;
    DECENT=false;
    RACnt--;                                 //decrement RA Counter;
    if(RACnt == 0){                          // If 0 then step motor
      RADirTmp=RADirFlag;
      if(RADirFlag)
      {
        RACnt=RASpeedsneg[RAnegSpeed];    // Reload Counter based of if it is driving Positive or Negative
        if(RAnegSpeed == 1)
          RADirTmp=!RADirTmp;
        if(RAturbo)
          RACnt=RASpeedsneg[5];
      }
      else
      {
        RACnt=RASpeedspos[RAposSpeed];
        if(RAturbo)
          RACnt=RASpeedspos[5];
      }
     if(RACnt != 0)                       
          RAENT=true;
    }
    if(DECenable){
      DECCnt--;                                 //decrement DEC counter for RA;
      if(DECCnt == 0){                          // If 0 then step motor
        DECCnt=DECSpeeds[DECSpeed];
        if(DECturbo)
           DECCnt=DECSpeeds[5];
        if(DECCnt != 0)
          DECENT=true;
  
      }
    }
    if(GoToEN)
    {
      //GoTo in progress decrement counters
      if(GoToRA > 0)
      {
          GoToRA--;                      //Decrement RA count
          RAp.pos=RAp.pos+RASstp;
          if(GoToRA==0)                   //Check to see if RA goto is finished
          {
            RAposSpeed= 0;                  //If so put RA back in normal mode
            RACnt=RASpeedspos[RAposSpeed];
            RADirFlag=RAStepDir;
            if(GoToDEC<=0)
            {
              Slewing=false;
              GoToEN=false;               //Go back to normal drive mode
            }
            RAp.pos=RAt.pos;
          }
      }
      if(GoToDEC > 0)
      {
          GoToDEC--;                      //Decrement RA count
          DECp.pos=DECp.pos+DECSstp;
          if(GoToDEC==0)                   //Check to see if RA goto is finished
          {
            DECenable=false;               // Turn off DEC
             if(GoToRA<=0)                  // If RA is finished also, shut down GOTO
            {
              Slewing=false;
              GoToEN=false;               //Go back to normal drive mode
            }
            DECp.pos=DECt.pos;
          }
      }
          
      
    }
    if(RAENT || DECENT)
    {
    DECDirTmp=DECDirFlag;
    if (!emStop) {
      if(!Configs.EQN)
      {
        RADirTmp=!RADirTmp;
        DECDirTmp=!DECDirTmp;
      }
      digitalWrite(RAdrvDIR, RADirTmp );
      digitalWrite(DECdrvDIR, DECDirTmp );
      digitalWrite(drvENABLE, LOW);
    }
    }
    HalfStep=true;
  }
   nowms=millis();                                 //get current time
   if(nowms-prevms > offms)
  {
    prevms=prevms+offms;
    seconds=seconds+1;
    if(HB)
    {
      HB2--;
      if(HB2==0)
      {
        digitalWrite(HBLED,LOW);
        HB=false;
        HB2=2;
        if(EastSide)
          HB2=1;
      }
    }
    else
    {
      HB2--;
      if(HB2==0)
      {
        digitalWrite(HBLED,HIGH);
        HB=true;
        HB2=2;
        if(EastSide)
          HB2=1;
      }
    }
    if(buzon)
        buzz(Buzzer,2500,200);

    if(seconds >59)
    {
      seconds=0;
      minutes=minutes+1;
      if(minutes >59)
      {
        minutes=0;
        hours=hours+1;
        if(hours > 23)
          hours=0;
      }
    }
  }

}


//**************************************************************************************
void clrLine0(){
  lcd.setCursor(0,0);
  lcd.print(clrLine);
  lcd.setCursor(0,0);
  
}
void clrLine1(){
  lcd.setCursor(0,1);
  lcd.print(clrLine);
  lcd.setCursor(0,1);
}

//**************************************************************************************
// Print status to LCD
void printStatus(uint8_t nobuzz){
  clrLine0();
  switch (Configs.guideSpeed){
    case 1:
      lcd.print(F(".5X"));
      break;
    case 2:
      lcd.print(F(" 2X"));
      break;
    case 3:
      lcd.print(F(" 4X"));
      break;
    case 4:
      lcd.print(F(" 8X"));
      break;
    case 5:
      lcd.print(F("16X"));
      break;
    default:
      lcd.print(F("ERR"));
  }
    if(emStop)
    {
      clrLine0();
      lcd.print("STP");
    }
    lcd.print(char(dotChar));                       // Print a dot
  if(drvRate == 0)
    lcd.print(F("Sid"));
  else if(drvRate == 1)
    lcd.print(F("Sol"));
  else
    lcd.print(F("Lun"));
    lcd.print(char(dotChar));                       // Print a dot
  if(EastSide)
    lcd.print(F("East"));
  else
    lcd.print(F("West"));
    lcd.print(char(dotChar));                       // Print a dot
  if(Configs.EQN)
    lcd.print(F("EQN"));
  else
    lcd.print(F("EQS"));
  clrLine1();
  lcd.print(hours);
  lcd.print(":");
  lead0(minutes,2);
  errPrint(nobuzz);
  MTimeOut=0;
  
}
//**************************************************************************************
void errPrint(uint8_t buzzn){
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  strcpy_P(buffer, (char*)pgm_read_word(&(ErrTable[errnum])));  // Copy to buffers
  lcd.print(" ");
  lcd.print(buffer);
  if(errnum==4)
    lcd.print(PCnum);
    lcd.print(PCcmd);
  if(buzzn==1)
    buzz(Buzzer,errFreq[errnum]*100,errLen[errnum]*100);
  
}
//**************************************************************************************
void itemPrint(int itm){
  long sftr=1;
  Serial.print("Item=");
  Serial.print(itm);
  // set the cursor to column 0, line 1
  // (note: line 1 is the second row, since counting begins with 0):
  if(itmPrnt)
  {
  clrLine1();
  strcpy_P(buffer, (char*)pgm_read_word(&(MenuTable[itm])));  // Copy to buffers
  if(Configs.itemEnable & sftr << itm)
    lcd.print(F("*"));
  else
    lcd.print(F(" "));
  
  lcd.print(buffer);
  }
  itmPrnt=true;
}
//**************************************************************************************
//  SETUP ROUTINE
//**************************************************************************************

void setup() {
// set direction and step to low
//  emSTOP=true;
  pinMode(EW1,INPUT_PULLUP);
  pinMode(HBLED, OUTPUT);

  pinMode(RAdrvDIR, OUTPUT);
  pinMode(RAdrvSTP, OUTPUT);
  pinMode(DECdrvDIR, OUTPUT);
  pinMode(DECdrvSTP, OUTPUT);
  pinMode( drvENABLE, OUTPUT );    // enable the driver board
  digitalWrite(drvENABLE, HIGH);                   // Disable motors
  digitalWrite( RAdrvDIR, 0 );
  digitalWrite( RAdrvSTP, 0 );
  digitalWrite( DECdrvDIR, 0 );
  digitalWrite( DECdrvSTP, 0 );
  
  Serial.begin(9600);
  // set up the LCD's number of columns and rows: 
  pinMode(Buzzer,OUTPUT);
  digitalWrite(Buzzer,LOW);               // Turn buzzer off
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  datasize = sizeof( Configs );
  nlocations = ((EEPROMSIZE-20) / datasize)-1;
  for (int lp1 = 0; lp1 < nlocations; lp1++ )
  {
    int addr = lp1 * datasize + 20;
    EEPROM_readAnything( addr, Configs );
    // check to see if the data is valid
    if ( Configs.Zfield == CurntConf && ForceConfigs == 0 )
    {
      // data was erased so write some default values
      currentaddr = addr;
      found = true;
    }
  }
  if ( found == true )
  {
    // set the controller back to the previous settings
    // done after this in one hit
    // mark current eeprom address as invalid and use next one
    // each time the controller starts it will read current storage, set it to invalid, goto next location and
    // write values to there and set it to valid - so it doesnt always try to use same locations over and
    // over and destroy the eeprom
    // using it like an array of [0-nlocations], ie 100 storage locations for 1k EEPROM
//    Serial.print("Found EEPROM Data at- ");
//    Serial.println(currentaddr);
    if(Configs.LatDeg != 0)                 //Check to see if there is a valid location
    {
      bitClear(TimeLocValid,2);             // Clear Lat bit in TimeLocValid flag
      bitClear(TimeLocValid,3);             // Clear Long bit in TimeLocValid flag
    }
    EEPROM_readAnything( currentaddr, Configs );
    Configs.Zfield = 0;
    writeEEPROMNow();    // update values in EEPROM
    // goto next free address and write data
    currentaddr += datasize;
    // bound check the eeprom storage and if greater than last index [0-EEPROMSIZE-1] then set to 0
    if ( currentaddr >= (nlocations * datasize ) ) currentaddr = 20;
    Configs.Zfield = CurntConf;
    writeEEPROMNow();    // update values in EEPROM
  }
  else
  {
    // set defaults because not found
//    Serial.println("Getting Default Data");
// Serial.println("Writing data to 20");
    currentaddr=20;
    defaultConfigs();
    writeMemory();
  }
  if(Configs.SidRate > 15000)
    drvScale[0]=Configs.SidRate;
  else
  {
    Configs.SidRate=18699;                //Sidereal Rate
    drvScale[0]=Configs.SolRate;
  }
  if(Configs.SolRate > 15000)
    drvScale[1]=Configs.SolRate;
  else
  {
    Configs.SolRate=18750;                //Solar Rate
    drvScale[1]=Configs.SolRate;
  }
  if(Configs.LunRate > 15000)
    drvScale[2]=Configs.LunRate;
  else
  {
    Configs.LunRate=19152;                //Lunar Rate
    drvScale[2]=Configs.LunRate;
  }
  drvScale[2]=Configs.LunRate;
  Configs.blank1=0;
  Configs.blank2=0;
  Configs.blank3=0;
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  lcd.begin(16, 2);
  lcd.setBacklight(Configs.blColor);
  lcd.blink();                            // set blinking cursor
  lcd.println("Board Setup");
  RAposSpeed=0;
  RAnegSpeed=0;
  RACnt=RASpeedspos[0];
  DECSpeed=0;
  DECCnt=DECSpeeds[0];
  RAStepDir=Configs.RAEQN;
  RADirFlag=RAStepDir;
  DECStepDir=Configs.DECEQN;
  DECDirFlag=DECStepDir;
  if(!EastSide)
    DECDirFlag=!DECStepDir;
  stepENON=1100;                 // delay for Enable to be on before Stepping
  stepENOFF=1100;                // delay for Enable to stay on after step
  stepontime=5;                 //time in microseconds that coil power is ON for one step, board requires 2us pulse
  DECturbo=false;
  RAturbo=false;

  timer1Setup();                           // set Interrupt Timer

//  RAp.pos= 0x3B300500;                             // Position of RA axis Mintaka
//  DECp.pos= 0xFFCB0500;                            // Position of DEC axis
  RAp.pos = 0x1F06EE00;                             // Position of RA axis Polaris
  DECp.pos= 0x3F879800;                             // Position of DEC axis
  errnum=0;
  printStatus(1);
}  // END SETUP

//**************************************************************************************
//  Main Loop
//**************************************************************************************
void loop() {
  buttons = lcd.readButtons();
  if (prevButton != buttons) {
    buzon=false;
    MTimeOut=millis()+60000;                                     // Time out for no buttons pressed in 1 minutes
    switch(inMenu){

      case 0:                                                   // 0 is Motor Control
        butPress=buttons;
        baseMenu();
      break;
      
      case 32:                                                  // 32 is the User menu
        butPress=buttons+inMenu;                                // Go to the User Menu
        UserMenu();
      break;
      default:
         ExtndMenu();                                           // Call extended menu
      break;
    }
  }

  recvWithCounter();          // Get Serial Data
  if(newData)                 //If Serial data is all read call routine to handle it
    PCParse();
  if(MTimeOut !=0 and MTimeOut < millis())     //Check for 1min. timeout on Menu, reset if timeout
    {
            // Button menu timeout. Clear menuflags and reset menu
            inMenu=0;
            MTimeOut=0;
            errnum=1;
            printStatus(1);
    }
  if(STimeOut !=0 and STimeOut < millis())     //Check for timeout on serial read, reset if timeout
    {
            // Serial data Time Out
            firstChar=true;
            newData = false;
            STimeOut=0;
            Serial.print("#");               // Send Acknoledgement char.
            if(SerMove> 0)
            {
              //turn it off
              MotStop(SerMove);
              SerMove=0;
            }
            errnum=4;
            printStatus(1);
   }

  if(!TimeSet)
      TimeDisp();
}
//*******************************************************************
// End Loop
//*******************************************************************

void TimeDisp()
{
  if(seconds != secPrev && inMenu == 0)
  {
    secPrev=seconds;
    Eval1=digitalRead(EW1);
//    Eval2=digitalRead(EW2);
    if(Eval1 != EvalPrev);
    {
        EvalPrev=Eval1;
        if(Eval1 == 1)
           EastSide=true;
        else
           EastSide=false;
      lcd.setCursor(8,0);
      if(EastSide)
        lcd.print("East");
      else
        lcd.print("West");
  
    }
    if(minutes != minPrev);                 //update time every minute
      lcd.setCursor(0,1);
      lcd.print(hours);
      lcd.print(":");
      lead0(minutes,2);
      lcd.setCursor(11,1);
      minPrev=minutes;
  }
}

//**************************************************************************************
// Buzzer Routine
//**************************************************************************************
// Inputs:
//         targetPin - pin on the Arduino the buzzer is attached to
//         frequency - frequency in hz. for the buzzer sound
//         length    - time in msec for the buzzer to sound

void buzz(int targetPin, long frequency, long length) 
{
  long delayValue = 1000000 / frequency / 2; // calculate the delay value between transitions
  //// 1 second's worth of microseconds, divided by the frequency, then split in half since
  //// there are two phases to each cycle
  long numCycles = frequency * length / 1000; // calculate the number of cycles for proper timing
  //// multiply frequency, which is really cycles per second, by the number of seconds to
  //// get the total number of cycles to produce
  if (Configs.enBUZZ) {
    for (long i = 0; i < numCycles; i++) { // for the calculated length of time...
      digitalWrite(targetPin, HIGH); // write the buzzer pin high to push out the diaphram
      delayMicroseconds(delayValue); // wait for the calculated delay value
      digitalWrite(targetPin, LOW); // write the buzzer pin low to pull back the diaphram
      delayMicroseconds(delayValue); // wait againf or the calculated delay value
    }
  }
}

//*********************************************************************************
//  This is to setup the values coming from Configs.  Will be the default for later
//*********************************************************************************
void defaultConfigs()
{
  Configs.Zfield=CurntConf;                    // If the first character is CurntConf that says the data has been written previously and is valid
  Configs.eeAccess=0;                   // Number of times the data has been accessed
  Configs.RAEQN=false;                  //Direction of RA step moves motor counter-clockwise in EQN, clockwise in EQS 
  Configs.DECEQN=false;                 //Direction of DEC step moves motor clockwise in EQN, counter-clockwise in EQS
  Configs.SidRate=18699;                //Sidereal Rate
  Configs.SolRate=18750;                //Solar Rate
  Configs.LunRate=19152;                //Lunar Rate
  Configs.enBUZZ=true;                  //Enable Buzzer 0-Buzzer Off  1-Buzzer allowed
  Configs.EQN=true;                     // True = EQ North  False=EQ South
  Configs.guideSpeed=5;                 // 1= .5x 2=2X, 3=4X, 4=8X, 5=16X, 6=40x (Slewing only)
  Configs.keys=0;                       // 0=Normal 1=Reverse RA 2= Reverse DEC 3=Reverse RA and DEC, 4=Flip RA/DEC 5=Flip+Rev RA, 6= Flip+ Rev DEC, 7=Flip+RevRA+RevDec
  Configs.blColor=1;                    //  1= Red, 2= Green, 3=Yellow, 4=Blue 5=Violet 6=Teal 7=White 8=Off
  Configs.LatDeg=0;
  Configs.LatMin=0;
  Configs.LatSec=0;
  Configs.LatDir=0;
  Configs.LongDeg=0;
  Configs.LongMin=0;
  Configs.LongSec=0;
  Configs.LongDir=1;
  Configs.itemEnable=itemEndef;
  TimeLocValid=15;
  EastSide=true;                        // Set scope to East side of the Meridian
  slewDelay=1500;                       // delay in usec between steps when slewing
  drvRate=0;                            // 0=Sidereal, 1=Solar 2=Lunar
  setGSflag();
  setBLflag();
  Configs.itemEnable=Configs.itemEnable & 0xFFFFF9FF;     //Disable Buzzer OFF
  if(Configs.enBUZZ)  
    Configs.itemEnable=Configs.itemEnable | 0x00000200;     //Set Buzzer On  
  else      
    Configs.itemEnable=Configs.itemEnable | 0x00000400;     //Set Buzzer Off        
  Configs.itemEnable=Configs.itemEnable & 0xFF3FFFFF;          //Clear out all EQN enable flags
  if(Configs.EQN)
    Configs.itemEnable=Configs.itemEnable | 0x00400000;          //Set EQN flag 
  else
    Configs.itemEnable=Configs.itemEnable | 0x00800000;          //Set EQS flag 
  writenow=true;                                                 //Set to write to memory
}
void setGSflag()
{
  Configs.itemEnable=Configs.itemEnable & 0xFFFFFFC0;                     //Clear the speed flags
  Configs.itemEnable=Configs.itemEnable | (1 << Configs.guideSpeed);      //Set .5x flag 
  writenow=true;
}

void setBLflag()
{
  uint32_t bits=0x00000800;
  Configs.itemEnable=Configs.itemEnable & 0xFFF807FF;                      //Clear out Back Light enable flags
  Configs.itemEnable=Configs.itemEnable | (bits << Configs.blColor);      //Set Back Light color flag 
  writenow=true;
}
void baseMenu()
{
  switch (butPress){
    case 0:
     //button released
//      lcd.clear();
//      lcd.home();
      RAturbo=false;
      DECturbo=false;
      MotStop(menuPrev);
      errnum=0;
      printStatus(0);
     break;
     case 1:
        buzz(Buzzer, 500, 200);     // buzz the buzzer at 500Hz for 500 milliseconds
        inMenu=32;                  //The enter button means to go into menu mode
        topPTR=0;                   //Point to first Menu Item
        subPTR=0;                   //Point to first item on the Menu list
        menuPTR=0;
        menuPrev=255;               // Set previous menu item to none 0
  //      dontExit=true;
        lcd.clear();
        lcd.home();     
        lcd.print(topMenu[topPTR]);
        itemPrint(0);
      break;
      
      case 2:   //RA West (right)key
      case 6:   //Right and Down Key pressed
      case 4:   //DEC Down Key
      case 8:   //DEC UP key
      case 10:  //Right and UP key pressed
      case 16:  //RA East (left) key
      case 20:  //Left and Down key pressed
      case 24:  //Left and UP key pressed
         MotMove( butPress,Configs.guideSpeed);                  // Move mount based on the button press (butPress) and Configs.guideSpeed
         guideSpdSav=Configs.guideSpeed;
      break;
      case 18:
       //Both RA keys pressed
      RAturbo=true;
      DECturbo=false;
      break;
      
      case 12:
       //Both DEC keys pressed
      RAturbo=false;
      DECturbo=true;
      break;
    }
    prevButton=buttons;
    menuPrev=butPress;
  }
//*********************************************************************************
//  Menu System 
//*********************************************************************************
void UserMenu()
{
  switch (butPress){

    case 32:
    // Do nothing, this is just a button unpress
//          if(dontExit)
//            dontExit=false;
    break;
    case 33:
    //Enter
      switch(menuPTR){
        case 0:
          inMenu=0;
          if(writenow)
            writeMemory();
//            errnum=0;
//          printStatus(1);
         break;
        case 1:
          Configs.guideSpeed=1;
          setGSflag(); 
        break;
        case 2:
          Configs.guideSpeed=2;
          setGSflag(); 
        break;
        case 3:
          Configs.guideSpeed=3;
          setGSflag(); 
        break;
        case 4:
          Configs.guideSpeed=4;
          setGSflag(); 
        break;
        case 5:
          Configs.guideSpeed=5;
          setGSflag(); 
        break;
        case 6:
        case 7:
        case 8:
          driveRate(255-menuPTR+6);              // Set drive rate
        break;

        case 9:
         //Set Buzzer ON
          Configs.enBUZZ=true;
          Configs.itemEnable=Configs.itemEnable & 0xFFFFF9FF;     //Disable Buzzer OFF
          Configs.itemEnable=Configs.itemEnable | 0x00000200;     //Set Buzzer ON        
          writenow=true;
        break;
        case 10:
          //Set Buzzer OFF
          Configs.enBUZZ=false;
          Configs.itemEnable=Configs.itemEnable & 0xFFFFF9FF;     //Disable Buzzer OFF
          Configs.itemEnable=Configs.itemEnable | 0x00000400;     //Set Buzzer ON        
          writenow=true;
        break;
        case 11:
         // backlight Off
          Configs.blColor=0;
          setBLflag(); 
          lcd.setBacklight(Configs.blColor);
        break;
        case 12:
         // RED backlight
          Configs.blColor=1;
          setBLflag(); 
          lcd.setBacklight(Configs.blColor);
        break;
        case 13:
         // Green backlight
          Configs.blColor=2;
          setBLflag(); 
          lcd.setBacklight(Configs.blColor);
        break;
        case 14:
         // Yellow backlight
          Configs.blColor=3;
          setBLflag(); 
          lcd.setBacklight(Configs.blColor);
        break;
        case 15:
         // Blue backlight
          Configs.blColor=4;
          setBLflag(); 
          lcd.setBacklight(Configs.blColor);
        break;
        case 16:
         // Violet backlight
          Configs.blColor=5;
          setBLflag(); 
          lcd.setBacklight(Configs.blColor);
        break;
        case 17:
         // Teal backlight
          Configs.blColor=6;
          setBLflag(); 
          lcd.setBacklight(Configs.blColor);
        break;
        case 18:
         // White backlight
          Configs.blColor=7;
          setBLflag(); 
          lcd.setBacklight(Configs.blColor);
        break;
        
        case 19:
          //Sidereal Time
          inMenu=12;
        break;
        
        case 20:
          //Set Location
          LatLong=true;
          SetLatitude();
        break;
        
        case 21:
          //Set Time/Date
          TimeDate=true;
          SetTime();
        break;
        
        case 22:
         setEQN();
        break;
        case 23:
          setEQS();
        break;
        case 24:
         //Reset Defaults
         inMenu=15;
        break;

        case 25:
         //Display Version
         inMenu=14;
        break;

        case 26:
          //Enter Coords
          RaDec=true;
          ValP.pos=RAp.pos;
          ValP2int(24, &RAHrs, &RAMin, &RASec);
          SetRA();

        break;
        case 27:
          inMenu=13;
        break;
        case 28:
          //GoTo
          inMenu=11;
        break;
        case 29:
          //Stop Motors
          if(emStop)
          {
            emStop=false;
            bitClear(Configs.itemEnable,FLStopMotors);
          }
          else
          {
            GoToEN=false;                   //Go back to normal drive mode
            emStop=true;
          // Stop any slewing and reset motor info
            GoToRA=0;                       //Clear RA count
            RAposSpeed= 0;                  //Put RA back in normal mode
            RACnt=RASpeedspos[RAposSpeed];
            RADirFlag=RAStepDir;
            GoToDEC=0;                      // Put DEC back in normal mode
            DECenable=false;
            Slewing=false;                  // Clear Slewing flag
            bitSet(Configs.itemEnable,FLStopMotors);
         }
        break;
        
       case 30:
          //
        break;

        break;
 //       default:
        
      }
      itemPrint(menuPTR);
      menuPrev=menuPTR;
      prevButton = buttons;
      guideSpdSav=Configs.guideSpeed;                             //Save current Configs.guideSpeed
      break;
    case 34:
      //Right
        topPTR++;                   //Point to first Menu Item
        if(topPTR >= topMenuMax)           //If reached max reset to 0 to roll around
          topPTR=0;
        subPTR=0;                         //Point to first item on the Menu list
        menuPrev=menuPTR;                       // Set previous menu item to none 0
        menuPTR=subMenuStart[topPTR];                        // set menuPRT
        lcd.clear();
        lcd.home();
        lcd.print(topMenu[topPTR]);
        itemPrint(menuPTR);
      break;
    case 36:
    //Down
        subPTR++;                               //Point to first item on the Menu list
        if(subPTR >= subMenuCnt[topPTR])                          //If reached max reset to 0 to roll around
          subPTR=0;
        menuPrev=menuPTR;                       // Set previous menu item to none 0
        menuPTR=subMenuStart[topPTR]+subPTR;    // set menuPRT
        itemPrint(menuPTR);
      break;
    case 40:
    //UP
        subPTR--;                               //Point to first item on the Menu list
        if(subPTR > subMenuCnt[topPTR])                          //If reached max reset to 0 to roll around
          subPTR=subMenuCnt[topPTR]-1;
        menuPrev=menuPTR;                       // Set previous menu item to none 0
        menuPTR=subMenuStart[topPTR]+subPTR;    // set menuPRT
        itemPrint(menuPTR);
     break;
    case 48:
    //Left
        topPTR--;                   //Point to first Menu Item
        if(topPTR > topMenuMax)           //If reached max reset to 0 to roll around
          topPTR=topMenuMax-1;
          
        subPTR=0;                         //Point to first item on the Menu list
        menuPrev=menuPTR;                       // Set previous menu item to none 0
        menuPTR=subMenuStart[topPTR];                        // set menuPRT
       lcd.clear();
        lcd.home();
        lcd.print(topMenu[topPTR]);
        itemPrint(menuPTR);
      break;
    default:
       buzz(Buzzer, 200, 500); // buzz the buzzer at 200Hz for 500 milliseconds
       errnum=2;
       printStatus(1);
  }
    prevButton=buttons;
    menuPrev=butPress;
}
void ResetMenu(uint8_t mp, uint8_t tp, uint8_t sp)
{
    inMenu=32;
    subPTR=0;                         //Point to first item on the Menu list
    menuPTR=mp;                        // set menuPRT
    topPTR=tp;
    subPTR=sp;
    lcd.clear();
    lcd.home();
    lcd.print(topMenu[topPTR]);
    itemPrint(menuPTR);

}
void SetMenu(uint8_t num1, uint8_t* valA, uint8_t* valMin, uint8_t* valSec, uint8_t* valDir)
{
  int LFLt[]{-11,0,0,0,1,0,0,1,0,0,1,0,-11};
  uint8_t valmax1[]{0,24,12,90,180,24,90};
  uint8_t valmax2[]{0,59,31,59,59,59,59};
  uint8_t valmax3[]{0,59,99,59,59,59,59};
  uint8_t valmax4[]{0,1,12,1,1,1,1};
  uint8_t valminn[]{0,0,1,0,0,0,0};
  int valmin4[]{0,0,-12,0,0,0,0};
  int ValA;
  int ValMin;
  int ValSec;
  int ValDir;
  ValA=*valA;
  ValMin=*valMin;
  ValSec=*valSec;
  ValDir=*valDir;
  if(ValDir > 128)
     ValDir=ValDir-256;
   if(num1==2){
    LFLt[1]=-10;
    LFLt[12]=-10;
  }
  if(inMenu == 2){
    LFLt[1]=-10;
    LFLt[12]=-10;
  }
  

  prevButton=buttons;
  
  switch(buttons){
  case 1:
    *valA=ValA;
    *valMin=ValMin;
    *valSec=ValSec;
    *valDir=lowByte(ValDir);

    // Save and return
    switch(inMenu)
    {
      case 1:
        prevms=millis();
        secPrev=seconds;
        minPrev=minutes;
        TimeSet=false;                              //time set is finished
        bitClear(TimeLocValid,0);             // Clear time bit in TimeLocValid flag
        if(TimeDate || InitAll){
          TimeDate=false;
          SetDate();
        }
        else
        {
        inMenu=0;
//        errnum=0;
        printStatus(1);
        }
      break;
      
      case 2:
        // Date Set Complete
        bitClear(TimeLocValid,1);             // Clear Date bit in TimeLocValid flag
        if(InitAll){
          SetLatitude();
        }
        else
        {
          ResetMenu(FLSidTime,4,0);
        }
      break;
      
      case 3:
       // Latitude Set
        bitClear(TimeLocValid,2);             // Clear Lat bit in TimeLocValid flag
        if(LatLong || InitAll ){
          LatLong=false;
          InitAll=false;
          SetLongitude();
        }
        else
        {
        inMenu=0;
//        errnum=0;
        printStatus(1);
        }
      break;
      
      case 4:
       // Longitude Set
        bitClear(TimeLocValid,3);             // Clear Long bit in TimeLocValid flag
          ResetMenu(FLSetTimeDate,4,2);
      break;
      
      case 5:
       // RA Set
          Int2ValP(24, RAHrs, RAMin, RASec);
          RAt.pos=ValP.pos;
           ValP.pos=DECp.pos;
           DECValP2int(360, &DECDeg, &DECMin, &DECSec, &DECsgn);
          SetDEC();
       break;
      case 6:
       // DEC Set
          DECInt2ValP(360, DECDeg, DECMin, DECSec, DECsgn);
          DECt.pos=ValP.pos;
          ResetMenu(FLEnterCoords,5,0);
       break;
      
      default:
        inMenu=0;
//        errnum=0;
        printStatus(1);
        
    }
  break;
  
  case 2:
         LineFlag=LineFlag+1;
         while(LFLt[LineFlag] != 0)
         {
             LineFlag=LineFlag+LFLt[LineFlag];
         }
         lcd.setCursor(LineFlag,1);
  break;
  
  case 4:
    switch(LineFlag){
      case 1:
         ValA= ValA-100;
        if( ValA<valminn[inMenu])
           ValA= ValA+100;
       break;
      case 2:
        ValA= ValA-10;
        if( ValA < valminn[inMenu])
           ValA=valminn[inMenu];
      break;
      case 3:
         ValA= ValA-1;
        if( ValA < valminn[inMenu])
           ValA=valminn[inMenu];
       break;
      case 5:
         ValMin= ValMin-10;
        if( ValMin < valminn[inMenu])
           ValMin=valminn[inMenu];
      break;
      case 6:
         ValMin= ValMin-1;
        if( ValMin < valminn[inMenu])
           ValMin=valminn[inMenu];
      break;
      case 8:
         ValSec= ValSec-10;
        if( ValSec < valminn[inMenu])
           ValSec=valminn[inMenu];
      break;
      case 9:
         ValSec= ValSec-1;
        if( ValSec < valminn[inMenu])
           ValSec=valminn[inMenu];
      break;
      case 11:
         ValDir= ValDir-1;
        if( ValDir < valmin4[inMenu])
           ValDir=valmin4[inMenu];
      break;
    }
  break;
  
  case 8:
    switch(LineFlag){
      case 1:
         ValA= ValA+100;
        if( ValA>valmax1[inMenu])
           ValA=valmax1[inMenu];
       break;
      case 2:
         ValA= ValA+10;
        if( ValA>=valmax1[inMenu])
           ValA=valmax1[inMenu];
      break;
      case 3:
         ValA= ValA+1;
        if( ValA>=valmax1[inMenu])
           ValA=valmax1[inMenu];
      break;
      case 5:
         ValMin= ValMin+10;
        if( ValMin> valmax2[inMenu])
           ValMin=valmax2[inMenu];
      break;
      case 6:
         ValMin= ValMin+1;
        if( ValMin>valmax2[inMenu])
           ValMin=valmax2[inMenu];
      break;
      case 8:
         ValSec= ValSec+10;
        if( ValSec>valmax3[inMenu])
           ValSec=valmax3[inMenu];
      break;
      case 9:
         ValSec= ValSec+1;
        if( ValSec>valmax3[inMenu])
           ValSec=valmax3[inMenu];
      break;
      
      case 11:
         ValDir= ValDir+1;
        if( ValDir>valmax4[inMenu])
           ValDir=valmax4[inMenu];
      break;
    }
  break;
  
  case 16:
         LineFlag=LineFlag-1;
         while(LFLt[LineFlag] != 0)
         {
             LineFlag=LineFlag-LFLt[LineFlag];
         }
         lcd.setCursor(LineFlag,1);
  break;
  
  }
    *valA=ValA;
    *valMin=ValMin;
    *valSec=ValSec;
    *valDir=lowByte(ValDir);

}
void ExtndMenu()
{
    switch(inMenu){
      case 1:
      //Time menu
        ValPrint(1,1,2,0,hours,minutes,seconds);
        SetMenu(2,&hours, &minutes, &seconds,&DST);
      break;

      case 2:
      //Date Menu
        ValPrint(2,1,2,0,month,day,year);
        SetMenu(2,&month, &day, &year, &TZ);
        errnum=0;
      break;
      
      case 3:
      //Latitude Menu
        ValPrint(3,1,2,0,Configs.LatDeg, Configs.LatMin, Configs.LatSec);
        SetMenu(2,&Configs.LatDeg, &Configs.LatMin, &Configs.LatSec,&Configs.LatDir);
      break;
      
      case 4:
      //Longitude Menu
        ValPrint(4,1,1,0,Configs.LongDeg,Configs.LongMin,Configs.LongSec);
        SetMenu(3,&Configs.LongDeg, &Configs.LongMin, &Configs.LongSec,&Configs.LongDir);
      break;
      
      case 5:
      //Enter RA Menu
        SetMenu(2,&RAHrs, &RAMin, &RASec,&DECsgn);
        ValPrint(5,1,2,0,RAHrs,RAMin,RASec);
      break;

      case 6:
      //Enter DEC Menu
        ValPrint(6,1,2,0,DECDeg,DECMin,DECSec);
        SetMenu(2,&DECDeg, &DECMin, &DECSec,&DECsgn);
      break;

      case 11:
      // Goto return
        printValPT(1);
        switch(buttons){
          case 1:
            GoTo();
            if(errnum == 8)
            {
//               ResetMenu(FLSetTimeDate,4,2);
              InitAll=true;
              SetTime();
            }
            else 
               ResetMenu(FLGoto,5,2);
          break;
          case 2:
          case 4:
          case 8:
          case 16:
            ResetMenu(FLGoto,5,2);
          break;
        }
      break;
      case 12:
          SiderealTime();
          if(errnum == 8)
          {
              InitAll=true;
              SetTime();
          }
          else
          {
          lcd.clear();
          lcd.print("Time: ");
          ValPrint(7,0,6,1,hours,minutes,seconds);
          lcd.setCursor(0,1);
          lcd.print("Sid.:");
          ValPrint(7,1,6,1,SidHrs,SidMin,SidSec);
            if(buttons==1)
              ResetMenu(FLSidTime,4,0);
          }
      break;

      case 13:
      //Sync return
        printValPT(0);
        clrLine0();
        lcd.setCursor(0,0);
        lcd.print(F("Enter to Sync"));
        switch(buttons){
          case 1:
            RAp.pos=RAt.pos;
            DECp.pos=DECt.pos;
            ResetMenu(FLSync,5,1);
          break;
          case 2:
          case 4:
          case 8:
          case 16:
            ResetMenu(FLSync,5,1);
        }
      break;
      
      case 14:
      // Display version
      clrLine1();
        lcd.print(VerNum);
        switch(buttons){
          case 1:
          case 2:
          case 4:
          case 8:
          case 16:
            ResetMenu(FLDispVer,4,6);
          break;
        }
        break;
      case 15:
      // Reset Defaults
      clrLine1();
        lcd.print(F("Enter=Defaults"));
        switch(buttons){
          case 1:
            defaultConfigs();
            writenow=true;
            ResetMenu(FLResetdefaults,4,5);
          break;
          case 2:
          case 4:
          case 8:
          case 16:
            ResetMenu(FLResetdefaults,4,5);
          break;
        }

    }
    prevButton=buttons;
  }
void recvWithCounter() {
    static boolean recvInProgress = false;
    static uint8_t ndx = 0;
    uint8_t rc;
 //recvInProgress is false until after receive first byte. First byte determines the number of characters to follow
 // if (Serial.available() > 0) {
    while (Serial.available() > 0 && newData == false) {
        rc = Serial.read();
        if(firstChar)
         {
          if(rc > 62 && rc < 123)
          {
            numChars=slength[rc-63];                 //Get number of charcters to read
            PCcmd=rc;                                 // put command in byte for processing by PCParse
            recvInProgress = true;
            firstChar=false;
            ndx=0;
            PCnum++;
            STimeOut=millis()+60;
          }
          else
          {
            numChars=0;
            recvInProgress = false;
            firstChar=true;
            ndx = 0;
            newData = false;
            STimeOut=0;
          }
         }
          if (recvInProgress == true && numChars > 0) {
              if (ndx < numChars) {
                  receivedChars[ndx] = rc;
                  ndx++;
                  if (ndx == numChars) {
                    recvInProgress = false;
                    firstChar=true;
                    ndx = 0;
                    newData = true;
                    STimeOut=0;
                 }
              }
              else
              {
                  recvInProgress = false;
                  ndx = 0;
                  firstChar=true;
                  newData = false;
                  STimeOut=0;
              }
          }
      }
}

void PCParse() {
char buf[9];
  if(Debug)
    ChrPrint();                                              // If in debug mode print out input buffer
  firstChar=true;                                           // Reset the flags to allow for another command
  newData = false;
  STimeOut=0;
  switch(PCcmd){
    case 63:
      // ignore ?,
      
    break;
    case 66:
      // ignore B, goto ASM-ALT
    break;

    case 69:
      // E command Get RA/DEC, note 16bit values in ascii- example "34AB,12CE#"
        PrintHEX(RAp.pos, 4);
      Serial.print(",");
        PrintHEX(DECp.pos, 4);
      Serial.print("#");
    break;
    
    case 72:
     // H command Set Time
      seconds=receivedChars[3];
      minutes=receivedChars[2];
      hours=receivedChars[1];
      month=receivedChars[4];
      day=receivedChars[5];
      year=receivedChars[6];
      TZ=receivedChars[7];                                                         // Time Zone
       DST=receivedChars[8];
      Serial.print("#");
      TimeLocValid=TimeLocValid & 0x0C;             // Clear time and date set flags
    break;

    case 74:
      // J command Is Alignment Complete?
        if(TimeLocValid!=0)
          Serial.print(char(0));                 //If Time/Loc not valid,Response is a 0 (0x30);
      else 
          Serial.print(char(1));                 //Response is a 1 (0x31);
      Serial.print("#");
    break;

    case 75:
      // K command (echo byte)
      Serial.print(char(receivedChars[1]));
      Serial.print("#");
    break;
    
    case 76:
      // L command (Slewing Complete 0-Yes, 1-No)
      if(Slewing)
        Serial.print(1);
      else
        Serial.print(0);
      Serial.print("#");
    break;

    case 77:
     // M Command- Stop GoTo
      Configs.guideSpeed=guideSpdSav;
      MotStop(inbutsav);
      Serial.print("#");
     
    break;

    case 78:
      // N command
      Serial.print("Sidereal:");
      Serial.print(drvScale[0]);
      Serial.print(" Solar:");
      Serial.print(drvScale[1]);
      Serial.print(" Lunar:");
      Serial.println(drvScale[2]);

    break;
    
    case 79:
      // O command
      tempI=String2int(4);
      // Set Pos Guide Rate  receivedChars[5] has rate: FF= Sidereal  FE=Solar  FD= Lunar
      if(tempI < 15000 || tempI > 25000)
      {
        Serial.print("Drive Rate Value not in range (15000-25000)you sent:");
        Serial.println(tempI);
      }
      else
      {
        switch (receivedChars[2])
        {
          case 48:
            drvScale[0]=tempI;
            Configs.SidRate=tempI;
            Serial.print("Sidereal Rate changed to: ");
            driveRate(-1);
           break;
          case 49:
            drvScale[1]=tempI;
            Serial.print("Solar Rate changed to: ");
            Configs.SolRate=tempI;
            driveRate(-2);
          break;
          case 50:
            drvScale[2]=tempI;
            Serial.print("Lunar Rate changed to: ");
            Configs.LunRate=tempI;
            driveRate(-3);
          break;
  
          default:
          Serial.print("Drive Rate type undefined. You sent: ");
          Serial.print(char(receivedChars[2]));
          Serial.print(" Rate value: ");
        }
         Serial.println(tempI);
         writeMemory();
          errnum=3;
          printStatus(1);
      }
    break;
    
    case 80:
      // P command
      Pcmd();                                                 //Call P command Routine
    break;
    
    case 82:
      // R command  GoTo
      RAt.wrd[1]=String2int(1);
      RAt.wrd[0]=0;
      
      DECt.wrd[1]= String2int(6);
      DECt.wrd[0]= 0;
      Serial.print("#");
      GoTo();
    break;

    case 114:
      // r command  GoTo Precise
      RAt.wrd[1]= String2int(1);
      RAt.wrd[0]= String2int(5);
      
      DECt.wrd[1]= String2int(10);
      DECt.wrd[0]= String2int(14);
      Serial.print("#");
      GoTo();
    break;
    
    case 83:
      // S command  Sync
      RAp.wrd[1]=String2int(1);
      RAp.wrd[0]=0;
      
      DECp.wrd[1]= String2int(6);
      DECp.wrd[0]= 0;
      Serial.print("#");
    break;

    case 115:
      // s command  Sync Precise
      RAp.wrd[1]= String2int(1);
      RAp.wrd[0]= String2int(5);
      
      DECp.wrd[1]= String2int(10);
      DECp.wrd[0]= String2int(14);
      Serial.print("#");
    break;

    case 84:
      // T command
      if(receivedChars[1] ==2)                              //Second byte has mode  Ignore 0-Off and 1-Alt/Az
        setEQN();                                         // if 2 set EQN
      if(receivedChars[1] ==3)
        setEQS();                                         // if 3 set EQS
      Serial.print("#");
      errnum=3;
      printStatus(1);
    break;

    case 86:
      // V command, Get Version Number  Return 4 20, 4.2
      Serial.print(char(4));
      Serial.print(char(20));
      Serial.print("#");
    break;

    case 101:
      // e command-Get precise RA/DEC, note 32bit values in ascii- example "34AB0500,12CE0500#" only upper 24bits are used, lower 8 are 00
        PrintHEX(RAp.pos, 8);
      Serial.print(",");
        PrintHEX(DECp.pos, 8);
      Serial.print("#");
      
    break;

    case 104:
     // h command Get Time
      Serial.print(char(hours));
      Serial.print(char(minutes));
      Serial.print(char(seconds));
      Serial.print(char(month));
      Serial.print(char(day));
      Serial.print(char(year));
      Serial.print(char(TZ));
      Serial.print(char(DST));
    break;
    
    case 108:
      // l command-Get Local Sidereal Time
      SiderealTime();                //Calculate LSMT
        PrintHEX(SidTime.pos, 8);
        Serial.print("#");
      
    break;
    
    case 109:
      // m command-Get Model #.  Return 14-CGEM
      Serial.print(char(14));
      Serial.print("#");
      
    break;
    
    case 112:
     // p command- Get pier side returns E for east or W for west
     if(EastSide)
       Serial.print("E#");
     else
       Serial.print("W#");
    break;
    
    case 116:
      // t command-Get tracking mode
      if(Configs.EQN)
        Serial.print(char(2));              //2 is EQ north
      else
        Serial.print(char(3));              //3 is EQ south, these are the only responses to send
      Serial.print("#");
      
    break;
    
    case 118:
      //v command is unknown, do not respond
      Serial.print("#");
    break;
    
    case 87:
      // W command -Set Location
      Configs.LatDeg=receivedChars[1];
      Configs.LatMin=receivedChars[2];
      Configs.LatSec=receivedChars[3];
      Configs.LatDir=receivedChars[4];
      Configs.LongDeg=receivedChars[5];
      Configs.LongMin=receivedChars[6];
      Configs.LongSec=receivedChars[7];                                                         // Time Zone
      Configs.LongDir=receivedChars[8];
      Serial.print("#");
       TimeLocValid=TimeLocValid & 0x03;             // Clear lat and long set flags
     
     break;
     
    case 119:
      // w command -Get Location
      Serial.print(char(Configs.LatDeg));
      Serial.print(char(Configs.LatMin));
      Serial.print(char(Configs.LatSec));
      Serial.print(char(Configs.LatDir));
      Serial.print(char(Configs.LongDeg));
      Serial.print(char(Configs.LongMin));
      Serial.print(char(Configs.LongSec));
      Serial.print(char(Configs.LongDir));
      Serial.print("#");
      
    break;

}
    
 }

 void ChrPrint(){
    char Chrdat;
    
    Serial.print(F("PC Command= "));
    Chrdat=PCcmd;
    Serial.print(Chrdat);
    Serial.print(F(" Data= "));
    for( int cnt=1; cnt < numChars; cnt++){
      Chrdat=receivedChars[cnt];
    Serial.print(receivedChars[cnt],HEX);
    Serial.print(" ");
    }
    Serial.println();
    Serial.print(F("                     "));
    for( int cnt=1; cnt < numChars; cnt++){
      Chrdat=receivedChars[cnt];
    Serial.print(Chrdat);
    Serial.print(F("  "));
    }
    Serial.println();

 }

 void Pcmd(){
   int tmp;
   uint8_t btt;
   uint8_t movrat;

   // receivedChars[] definition for P cmd:
   //              [0]=0x50 "P"
   //              [1]=0x50 Message Length
   //              [2]=0x50 Device ID: 0x01=Main/Interconnection 0x04=HC 0x10 RA board 0x11 DEC board 0xB0 GPS board
   //              [3]=0x50 Message ID: 01 Get Position 02 gotoFast 04 Set Postion  06 Set Pos Guide Rate 07 set Neg Guide Rate 
   //                  24(36):move Pos Rate 25(37):move Neg Rate 46(70): Set Autoguide Rate  47(71): Get Autoguide Rate FE(254):Get Version
   //              [4]=0x50 Data1  
   //              [5]=0x50 Data2
   //              [6]=0x50 Data3
   //              [7]=number of bytes in return message   
   switch(receivedChars[3]){                                         // Divide based on Message ID

    case 1:
        // Get Position
        
    break;

    case 2:
        // GOTO_Fast
        
    break;

    case 4:
        // Set Position
        
    break;

    case 6:
        // Set Pos Guide Rate  receivedChars[5] has rate: FF= Sidereal  FE=Solar  FD= Lunar 
        driveRate(receivedChars[5]);              // Set drive rate
        Serial.print("#");                        // Send # response
        errnum=3;
        printStatus(1);
    break;

    case 7:
        // Set Neg Guide Rate
        
    break;


    case 36:
        // 0x24 Move Positive. Rate is in receivedChars[4] 0=Stop 1=.5x 2=2x 3=4x 5= 16x (6-9)=50x
        // receivedChars[2] = 16 for RA  17 for DEC
        movrat=receivedChars[4];
        if( movrat > 6)
            movrat=6;
        btt=2;
        if(receivedChars[2] == 17)
           btt=8;
        if(movrat == 0){
            Configs.guideSpeed=guideSpdSav;
            MotStop(inbutsav);
            SerMove=0;
        }
        else{
          guideSpdSav=Configs.guideSpeed;                             //Save Current guide speed
          Configs.guideSpeed=movrat;
          SerMove=btt;                                       // show we're in a serial move
          MotMove(btt, movrat);
        }
        inbutsav=btt;
        Serial.print("#");
    break;

    case 37:
        // 0x25 Move Negative Rate is in receivedChars[4] 0=Stop 1=.5x 2=2x 3=4x 5= 16x (6-9)=50x
        movrat=receivedChars[4];
        if( movrat > 6)
            movrat=6;
        btt=16;
        if(receivedChars[2] == 17)
           btt=4;
        if(movrat == 0){
            Configs.guideSpeed=guideSpdSav;
            MotStop(inbutsav);
            SerMove=0;
        }
        else{
         guideSpdSav=Configs.guideSpeed;                             //Save Current guide speed
         Configs.guideSpeed=movrat;
         SerMove=btt;                                       // show we're in a serial move
         MotMove(btt, movrat);
         SerMove=btt;

        }
           
        inbutsav=btt;
        Serial.print("#");
    break;

    case 70:
        // 0x46 Set Autoguide Rate Uses rate percentage formula: 100*receivedChars[4]/256
        if(receivedChars[2] == 16)
          RAGuideRate=receivedChars[4];
        else
          DECGuideRate=receivedChars[4];
        SetGuideRates();
        Serial.print("#");
    break;

    case 71:
        // 0x47 Get Autoguide Rate Uses rate percentage formula: 100*guiderate/256
        if(receivedChars[2] == 16)
          tmp=RAGuideRate;
        else
          tmp=DECGuideRate;
        Serial.print(char(tmp));
        Serial.print("#");
        
    break;

    case 254:
        // 0xFE Get Version Number MSB is major version #  LSB is minor version #
        Serial.print(char(6));
        Serial.print(char(8));
        Serial.print("#");
    break;

    default:
    lcd.clear();
    lcd.print(F("Unknown Messge "));
    lcd.print(receivedChars[3],HEX);
    
    
   }
 }

 void MotStop(uint8_t inbut){
        switch(inbut){
        case 2:
          RAStop();
          Slewing=false;
        break;

        case 4:
        case 8:
          DECenable=false;
          Slewing=false;
        break;
        
        case 16:
          RAStop();
          Slewing=false;
        break;
      }

 }
void RAStop()
{
          RAposSpeed= 0;
          RACnt=RASpeedspos[RAposSpeed];
          RADirFlag=RAStepDir;

}
void RAWest(uint8_t MotRate)
{
       // RA  (west)
      RAturbo=false;
      DECenable=false;
      RAposSpeed= MotRate;
      RACnt=RASpeedspos[MotRate];
      RADirFlag=RAStepDir;
      Slewing=true;

}
void DECSouth(uint8_t MotRate)
{
       //DEC Down (South)
      DECturbo=false;
      DECSpeed= MotRate;
      DECCnt=DECSpeeds[MotRate];
      if(EastSide)
        DECDirFlag=!DECStepDir;
      else
        DECDirFlag=DECStepDir;
      DECenable=true;
      Slewing=true;

}
void RAEast(uint8_t MotRate)
{
       //RA Positive (East)
      RAturbo=false;
      DECenable=false;
      RAnegSpeed= MotRate;
      RACnt=RASpeedsneg[MotRate];
      RADirFlag=!RAStepDir;
      Slewing=true;

}
void DECNorth(uint8_t MotRate)
{
       //Dec UP (North)
      DECturbo=false;
      DECSpeed= MotRate;
      DECCnt=DECSpeeds[MotRate];
      if(EastSide)
        DECDirFlag=DECStepDir;
      else
        DECDirFlag=!DECStepDir;
      DECenable=true;
      Slewing=true;
}
void MotMove(uint8_t MotDir, uint8_t MotRate)
{
   switch(MotDir){
     case 16:
       RAWest(MotRate);
       DECenable=false;
     break;
    case 4:
      RAStop();
      DECSouth(MotRate);
    break;
    case 6:
      RAEast(MotRate);
      DECSouth(MotRate);
    break;
    case 8:
      RAStop();
      DECNorth(MotRate);
    break;
    case 10:  //Right and UP key pressed
      RAEast(MotRate);
      DECNorth(MotRate);
    break;
    case 2:
      RAEast(MotRate);
      DECenable=false;
    break;
    case 20:  //Left and Down key pressed
      RAWest(MotRate);
      DECSouth(MotRate);
    break;
    case 24:  //Left and UP key pressed
      RAWest(MotRate);
      DECNorth(MotRate);
    break;

   }
}
void driveRate(int8_t ddRate)
{
  ddRate=-ddRate-1;
   switch(ddRate){
          case 0:
          drvRate=0;
          Configs.itemEnable=Configs.itemEnable & 0xFFFFFE3F;          //Clear out all enable flags
          Configs.itemEnable=Configs.itemEnable | 0x00000040;          //Set Sidereal rate flag 
          timer1Setup();                                                // Set drive rate
        break;
        case 1:
          drvRate=1;
          Configs.itemEnable=Configs.itemEnable & 0xFFFFFE3F;          //Clear out all enable flags
          Configs.itemEnable=Configs.itemEnable | 0x00000080;          //Set Solar rate flag 
          timer1Setup();                                                // Set drive rate
        break;
        case 2:
          drvRate=2;
          Configs.itemEnable=Configs.itemEnable & 0xFFFFFE3F;          //Clear out all enable flags
          Configs.itemEnable=Configs.itemEnable | 0x00000100;          //Set Lunar rate flag 
          timer1Setup();                                                // Set drive rate
        break;
   }
   if(inMenu==0){
//     errnum=0;
     printStatus(1);
   }
}
void setEQN(){
     //EQN
    Configs.EQN=true;
    Configs.itemEnable=Configs.itemEnable & 0xFF9FFFFF;          //Clear out all enable flags
    Configs.itemEnable=Configs.itemEnable | 0x00200000;          //Set EQN flag 
    writenow=true;
}
void setEQS(){
     //EQS
    Configs.EQN=false;
    Configs.itemEnable=Configs.itemEnable & 0xFF9FFFEF;          //Clear out all enable flags
    Configs.itemEnable=Configs.itemEnable | 0x00400000;          //Set EQN flag 
    writenow=true;
}

void SetGuideRates()
{
  int TempVar;
  TempVar= RASpeedspos[0]-((RAGuideRate*(RASpeedspos[0]-RASpeedspos[2]))/256);
  RASpeedspos[1]=TempVar;
  TempVar= RASpeedspos[0]*256/(256-RAGuideRate);
  RASpeedsneg[1]=TempVar;
  if(TempVar > 255)
    RASpeedsneg[0]=255;
  TempVar= RASpeedspos[0]-((DECGuideRate*(RASpeedspos[0]-RASpeedspos[2]))/256);
  DECSpeeds[1]=TempVar*3;
  
}
void SetTime(){
  TimeSet=true;
  lcd.clear();                                       // Clear the line and set up for Latitude set
  lcd.print(F("Set Time  DST?"));
  LineFlag=2;
  ValPrint(1,1,2,0,hours,minutes,seconds);
  inMenu=1;                                        // Set menu flag to show we're reading time
 // And we're done for now.
}

void SetDate(){
  lcd.clear();                                       // Clear the line and set up for Latitude set
  lcd.print(F("Set Date M/D/Y Z"));
  inMenu=2;                                        // Set menu flag to show we're reading time
  LineFlag=3;                                      // show we're pointing at Month
  ValPrint(2,1,2,0,month,day,year);
 // And we're done for now.
}

void SetLongitude(){
  lcd.clear();                                       // Clear the line and set up for Latitude set
  lcd.print(F("Set Longitude"));
  LineFlag=1;                                        // show we're pointing at degrees
  ValPrint(4,1,1,0,Configs.LongDeg,Configs.LongMin,Configs.LongSec);
  inMenu=4;                                        // Set menu flag to show we're reading time
 // And we're done for now.
  writenow=true;
}

void SetLatitude()
{
  lcd.clear();                                       // Clear the line and set up for Latitude set
  lcd.print(F("Set Latitude"));
  LineFlag=2;                                        // show we're pointing at degrees
  ValPrint(3,1,2,0,Configs.LatDeg,Configs.LatMin, Configs.LatSec);
  inMenu=3;                                        // Set menu flag to show we're reading time
 // And we're done for now.
  writenow=true;
}

void SetRA(){
  lcd.clear();                                       // Clear the line and set up for Latitude set
  lcd.print(F("Enter RA"));
  LineFlag=2;
  ValPrint(5,1,2,0,RAHrs,RAMin,RASec);
  inMenu=5;                                        // Set menu flag to show we're reading time
 // And we're done for now.
}

void SetDEC(){
  lcd.clear();                                       // Clear the line and set up for Latitude set
  lcd.print(F("Enter DEC"));
  LineFlag=2;
  ValPrint(6,1,2,0,DECDeg,DECMin,DECSec);
  inMenu=6;                                        // Set menu flag to show we're reading time
 // And we're done for now.
}

void lead0(int num, int8_t numchr)
{
    switch(numchr)
    {
      case 4:
        if(num<1000)
          lcd.print("0");
      case 3:
        if(num<100)
          lcd.print("0");
      case 2:
        if(num<10)
          lcd.print("0");
     }
     lcd.print(num);
}



void SiderealTime()
{
  float tempF1;
  float tempF2;
  long tempL;
  int TZtmp;
  int T1;
  int T2;
  int T3;
  if(TimeLocValid == 0)
  {
    T1=year+2000;
    T2=month;
    T3=day;
    TZtmp=TZ;
    if(TZ>128)
     TZtmp=TZtmp-256;
    tempF1=367.*T1-int(7.*(T1+int((T2+9)/12.))/4.)+int(275.*T2/9.)+T3-730531.5;    //dwhole
    T1=hours;
    T2=minutes;
    T3=seconds;
    tempF2=((T1-TZtmp-DST) + T2/60. + T3/3600.)/24.;                                      //dfrac
    tempF2+=tempF1;                                                               //d=dwhole+dfrac
    T1=Configs.LongDeg;
    T2=Configs.LongMin;
    T3=Configs.LongSec;
    tempF1=T1 + T2/60. + T3/3600.;
    if(Configs.LongDir == 1)
      tempF1=-tempF1;
    tempF2=(280.46061837 + 360.98564736629 * tempF2 +tempF1)/360.;
     tempF1=tempF2;
    tempF2=tempF2-int(tempF1);
    tempF1=tempF2;
    tempF1=(tempF2*4294967296.);
    SidTime.pos=tempF1;
    SidTime.wrd[0]=tempF1;
    tempF2=tempF2*24;
    SidHrs=int(tempF2);
    tempF2=(tempF2-SidHrs)*60;
    SidMin=int(tempF2);
    tempF2=(tempF2-SidMin)*60;
    SidSec=int(tempF2);
  }
  else
  {
    errnum=8;
    printStatus(1);
  }
}
void PrintHEX(unsigned long DATA, unsigned char numChars) {
    unsigned long mask  = 0xF0000000;
//    mask = mask << 4*(numChars-1);
    
    for (unsigned int i=8; i>8-numChars; --i) {
         Serial.print(((DATA & mask) >> (i-1)*4),HEX);
         mask = mask >> 4;
        }
 
}

void ValP2int(uint16_t mult, uint8_t* valA, uint8_t* valMin, uint8_t* valSec)
{
  //********************************************************************************************
  // To call this routine store the value to be converted in ValP.pos
  // ValP.pos is assumed to be a value 0-1 divided by 4294967296
  // mult is the value that ValP.pos is multiplied by to give the result.
  // For example, RAp.pos is multiplied by 24 to give Right Ascension, DECp.pos is multiplied by 360
  //              
  // Input: Value should be stored in the Union ValP.pos before the call
  // Returns:
  //     ValA = The Integer portion of the value.
  //     ValMin= The fracion of the value multiplied by 60 (usually Minutes)
  //     ValSec= The fracion of the value multiplied by 3600 (usually Seconds)
  //
  //  Note: Valtmp is also a union declared in the first part of this file
  //********************************************************************************************
  uint16_t tempL;
  uint16_t tempB;
  Valtmp.pos=0;                         // Clear temp variable
  tempL=ValP.wrd[1];                    //tempL = ValP.pos / 65536
  Valtmp.wrd[0]=tempL;        // Multiply mult by high word (save as divide by 65536)
  Valtmp.pos=Valtmp.pos * mult;        // Multiply mult by high word (save as divide by 65536)
  *valA=Valtmp.wrd[1];                   // By using the high word it is another divide by 65536 so total divide by 4294967296
  // valA now has the integer part of ValP.pos*mult/4294967296
  Valtmp.pos=0;                         // Clear the temp value
  Valtmp.wrd[1]=*valA;                        // multiply valA by 65536
  Valtmp.pos=Valtmp.pos/mult;                // divide by mult
  tempL=tempL-Valtmp.wrd[0];             // Subtract the value of valA*mult*4294967296
  tempB=tempL;
  Valtmp.pos=0;                          // Clear out temp value again
  Valtmp.pos=tempL * mult* 60.;          // multiply by mult again and 60
  *valMin=Valtmp.wrd[1];                  // divide by 65536 to get the Minutes Value
  Valtmp.wrd[0]=0;
  Valtmp.pos=Valtmp.pos/mult;
  Valtmp.pos=Valtmp.pos/60.;
  tempB=tempL;
  tempL=tempL-Valtmp.wrd[0];
  *valSec=tempL;                          //Should be seconds
   
}
void Int2ValP(int mult, int valA, int valMin, int valSec)
{
  //********************************************************************************************
  // ValP.pos is assumed to be a value 0-1 divided by 4294967296
  // mult is the value that ValP.pos is multiplied by to give the result.
  // For example, RAp.pos is multiplied by 24 to give Right Ascension, DECp.pos is multiplied by 360
  //
  // Input:  
  //     ValA = The Integer portion of the value.
  //     ValMin= The fracion of the value multiplied by 60 (usually Minutes)
  //     ValSec= The fracion of the value multiplied by 3600 (usually Seconds)
  //     ValDir= The sign of the value. 1=positive, -1= negative
  //              
  // Returns: Output value is stored in the Union ValP.
  //********************************************************************************************
  uint32_t Mx=0xFFFFFFFF;
   Mx=Mx/mult;
   Mx=Mx/3600;
   Valtmp.pos=(valA*60.+valMin);
   Valtmp.pos=Valtmp.pos*60.+valSec;
   ValP.pos=Valtmp.pos*Mx;
}

void DECValP2int(uint16_t mult, uint8_t* valA, uint8_t* valMin, uint8_t* valSec, uint8_t* neg)
{
  uint8_t vva;
  uint8_t vvb;
  uint8_t vvc;
//  ValP.pos=DECt.pos;
  *neg=0;
  if(ValP.pos & 0x80000000 )
  {
    ValP.pos=~ValP.pos;
    ValP.pos=ValP.pos+1;
    *neg=1;
  }
  vva=*valA;
  vvb=*valMin;
  vvc=*valSec;
  ValP2int(mult,&vva, &vvb, &vvc);
  *valA=vva;
  *valMin=vvb;
  *valSec=vvc;
}  

void DECInt2ValP(uint16_t mult, uint16_t valA, uint16_t valM, uint16_t valS, int neg)
{
    Int2ValP(mult,valA, valM, valS);
    if(neg==1 )
      {
        ValP.pos=~ValP.pos;
        ValP.pos=ValP.pos+1;
      }
  DECt.pos=ValP.pos;
  }
  
  uint32_t String2int(int pos)
{
    for(int cnt=pos; cnt<pos+4;cnt++)
    {
      iString[cnt-pos]= char(receivedChars[cnt]);
    }
    uint16_t val= strtol(iString,NULL,16);
    return val;
}


uint8_t side(long x, long y)
{
  long z;
  z=x-y;
  
//  Serial.print("x=");
//  Serial.println(x,HEX);
//  Serial.print("y=");
//  Serial.println(y,HEX);
//  Serial.print("Z=");
//  Serial.println(z,HEX);
  if(z > 0x20000000)
    z=z-0x40000000;
  if(z<0)
    return 1;
  else
    return 0;
}
void GoTo()
{
  long dist;
  long dist1;
  long dist2;
  uint32_t dtmp1;
  int quadp;
  int quadt;
  boolean RAdirs=true;
  boolean DECdirs=true;
  boolean quadroll = false;
  
  #define debugs false
  #define quadrant 0x40000000
  // RAt.pos and DECt.pos contain the location we want to go
  // RAp.pos and DECp.pos contain the location where we are
  // Calculate distance between RAt and RAp
  if(TimeLocValid==0)                                     //can only do a goto with a valid time
  {
    SiderealTime();   
    dtmp1=SidTime.pos>>1;
    dist=dtmp1;                                   // Check postions versus meridian
    dtmp1=RAt.pos;
    dtmp1=dtmp1 >> 1;
    dist1=dtmp1;
    dtmp1=RAp.pos;
    dtmp1=dtmp1 >> 1;
    dist2=dtmp1 ;
    if(side(dist1, dist) == side(dist2, dist))
    {                                                        // On the same side of meridian
      quadp=RAp.pos/quadrant;
      quadt=RAt.pos/quadrant;
      if((quadp ==3 && quadt == 0) || (quadp==0 && quadt == 3))
          quadroll=true;
      dist1=RAt.pos/5825;
      if(quadroll && quadt == 3)
         dist1=dist1-737333;
      dist2=RAp.pos/5825;
       if(quadroll && quadp == 3)
         dist2=dist2-737333;
      dist= dist1-dist2;
       GoToRA=abs(dist);
      if(dist <0)
        RAdirs=false;
      
      dist1=DECt.pos/17476;
      if(DECt.wrd[1] > 32767)
          dist1=dist1-245763;
      dist2=DECp.pos/17476;
      if(DECp.wrd[1] > 32767)
          dist2=dist2-245763;
      dist=dist1-dist2;
      
     GoToDEC=abs(dist);
     if(dist<0)
         DECdirs=false;
       
      if(GoToDEC>0)
      {
        DECturbo=false;
        DECSpeed= 6;
        DECCnt=DECSpeeds[6];
        DECDirFlag=!DECStepDir; 
        if(DECdirs != EastSide)
          DECDirFlag=DECStepDir;           
        DECenable=true;
        DECSstp=17476;
        if(!DECdirs)
          DECSstp=-17476;
      }
      if(GoToRA > 0)
      {
          RAturbo=false;
          if(RAdirs)
          {
           //RA Positive (East)
            RAnegSpeed= 6;
            RACnt=RASpeedsneg[6];
            RADirFlag=!RAStepDir;
            RASstp=5825;
          }
          else
          {
            RAposSpeed= 6;
            RACnt=RASpeedspos[6];
            RADirFlag=RAStepDir;
            RASstp=-5825;
          }
      }
        GoToEN=true;
        Slewing=true;
    }
  else
    {
    // If not on same side print an error message and beep buzzer
      errnum=7;
      printStatus(1);
    }
  }
 else
 {
  // If time isn't valid print an error message and beep buzzer
   errnum=8;

   printStatus(1);
 }
}
void printValPT(uint8_t r2p)
{
  if(r2p==1)
  {
    ValP.pos=RAp.pos;
    ValP2int(24,&RAHrs,&RAMin,&RASec);
    ValPrint(8,0,0,0,RAHrs,RAMin,RASec);
    ValP.pos=DECp.pos;
    DECValP2int(360,&DECDeg,&DECMin,&DECSec, &DECsgn );
    lcd.print(" ");
    ValPrint(9,0,7,1,DECDeg,DECMin,DECSec);
  }
  ValP.pos=RAt.pos;
  ValP2int(24,&RAHrs,&RAMin,&RASec);
  ValPrint(8,1,0,0,RAHrs,RAMin,RASec);
  ValP.pos=DECt.pos;
  DECValP2int(360,&DECDeg,&DECMin,&DECSec, &DECsgn );
  lcd.print(" ");
  ValPrint(9,1,7,1,DECDeg,DECMin,DECSec);

}
void ValPrint(uint8_t inval,uint8_t lnum,uint8_t cnum, uint8_t lclr, uint8_t val1, uint8_t val2, uint8_t val3)
{ 
  uint8_t Lval;
  // inval=1 Time, =2 Date, =3 Lat, =4 Long, =5 RA, =6 DEC
  if(lclr == 0)
  {
    if(lnum == 1)
      clrLine1();                                       // Clear the line and set up for time set
     else
      clrLine0();
  }
    lcd.setCursor(cnum,lnum);
  if(inval == 4)
    lead0(val1,3);
  else
    lead0(val1,2);
  if(inval == 2)
    lcd.print("/");
  else
    lcd.print(":");
  lead0(val2,2);
  if(inval < 8)
  {
    if(inval == 2)
      lcd.print("/");
    else
      lcd.print(":");
    if(inval==2)
      lead0(year+2000,4);
    else
        lead0(val3,2);
  }
  lcd.print(" ");
  switch(inval){
    case 1:
      if(DST==0)
        lcd.print("N");
      else
        lcd.print("Y");
      Lval=LineFlag;
    break;

    case 2:
      if(TZ<128)
      {
        lcd.print("+");
        lead0(abs(TZ),2);
      }
      else
      {
        lcd.print("-");
        lead0(256-TZ,2);
      }
      Lval=LineFlag;                             
      if(LineFlag >= 7)
        Lval=LineFlag+2;
    break;
    
    case 3:
       if(Configs.LatDir == 0)
        lcd.print("N");
      else
        lcd.print("S");
      Lval=LineFlag;                             
    break;
 
    case 4:
      if(Configs.LongDir == 0)
        lcd.print("E");
      else
        lcd.print("W");
      Lval=LineFlag;                             
     break;

    case 5:
      lcd.print(" ");
      Lval=LineFlag;                             
     break;
 
    case 6:
      if(DECsgn==0)
        lcd.print("+");
      else
        lcd.print("-");
      Lval=LineFlag;                             
     break;

     case 7:
       Lval=11;
     break;
       
    case 9:
      lcd.setCursor(6,lnum);
      if(DECsgn==0)
        lcd.print("+");
      else
        lcd.print("-");
      Lval=LineFlag;                             
     break;

  }
 lcd.setCursor(Lval,lnum);                              // Set cursor to correct position 
}

void writeMemory()
{
  Configs.Zfield = CurntConf;
  Configs.eeAccess++;
  writeEEPROMNow();
  writenow = false;
   
}
void writeEEPROMNow()
{
  Configs.EEPROMWrites++;
  EEPROM_writeAnything(currentaddr, Configs);    // update values in EEPROM
} // end of writeEEPROMNow()
